@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.builders.latex
     ~~~~~~~~~~~~~~~~~~~~~
@@ -13,32 +12,39 @@ import os
 from os import path
 
 from docutils.frontend import OptionParser
-from six import text_type
 
+import sphinx.builders.latex.nodes  # NOQA  # Workaround: import this before writer to avoid ImportError
 from sphinx import package_dir, addnodes, highlighting
 from sphinx.builders import Builder
 from sphinx.builders.latex.transforms import (
     BibliographyTransform, CitationReferenceTransform, MathReferenceTransform,
     FootnoteDocnameUpdater, LaTeXFootnoteTransform, LiteralBlockTransform,
-    ShowUrlsTransform, DocumentTargetTransform,
+    ShowUrlsTransform, DocumentTargetTransform, IndexInSectionTitleTransform,
 )
-from sphinx.config import string_classes, ENUM
+from sphinx.builders.latex.util import ExtBabel
+from sphinx.config import ENUM
 from sphinx.environment import NoUri
 from sphinx.environment.adapters.asset import ImageAdapter
-from sphinx.errors import SphinxError, ConfigError
+from sphinx.errors import SphinxError
 from sphinx.locale import _, __
 from sphinx.transforms import SphinxTransformer
-from sphinx.util import texescape, logging, status_iterator
+from sphinx.util import texescape, logging, progress_message, status_iterator
 from sphinx.util.console import bold, darkgreen  # type: ignore
 from sphinx.util.docutils import SphinxFileOutput, new_document
 from sphinx.util.fileutil import copy_asset_file
+from sphinx.util.i18n import format_date
 from sphinx.util.nodes import inline_all_toctrees
-from sphinx.util.osutil import SEP, make_filename
-from sphinx.writers.latex import DEFAULT_SETTINGS, LaTeXWriter, LaTeXTranslator
+from sphinx.util.osutil import SEP, make_filename_from_project
+from sphinx.util.template import LaTeXRenderer
+from sphinx.writers.latex import (
+    ADDITIONAL_SETTINGS, DEFAULT_SETTINGS, LaTeXWriter, LaTeXTranslator
+)
+
+# load docutils.nodes after loading sphinx.builders.latex.nodes
+from docutils import nodes  # NOQA
 
 if False:
     # For type annotation
-    from docutils import nodes  # NOQA
     from typing import Any, Dict, Iterable, List, Tuple, Union  # NOQA
     from sphinx.application import Sphinx  # NOQA
     from sphinx.config import Config  # NOQA
@@ -99,11 +105,11 @@ XINDY_LANG_OPTIONS = {
     'el': '-L greek -C utf8 ',
     # FIXME, not compatible with [:2] slice but does Sphinx support Greek ?
     'el-polyton': '-L greek -C polytonic-utf8 ',
-}  # type: Dict[unicode, unicode]
+}
 
 XINDY_CYRILLIC_SCRIPTS = [
     'be', 'bg', 'mk', 'mn', 'ru', 'sr', 'sh', 'uk',
-]  # type: List[unicode]
+]
 
 logger = logging.getLogger(__name__)
 
@@ -126,24 +132,29 @@ class LaTeXBuilder(Builder):
 
     def init(self):
         # type: () -> None
-        self.docnames = []          # type: Iterable[unicode]
-        self.document_data = []     # type: List[Tuple[unicode, unicode, unicode, unicode, unicode, bool]]  # NOQA
+        self.babel = None           # type: ExtBabel
+        self.context = {}           # type: Dict[str, Any]
+        self.docnames = []          # type: Iterable[str]
+        self.document_data = []     # type: List[Tuple[str, str, str, str, str, bool]]
         self.usepackages = self.app.registry.latex_packages
         texescape.init()
 
+        self.init_context()
+        self.init_babel()
+
     def get_outdated_docs(self):
-        # type: () -> Union[unicode, List[unicode]]
+        # type: () -> Union[str, List[str]]
         return 'all documents'  # for now
 
     def get_target_uri(self, docname, typ=None):
-        # type: (unicode, unicode) -> unicode
+        # type: (str, str) -> str
         if docname not in self.docnames:
             raise NoUri
         else:
             return '%' + docname
 
     def get_relative_uri(self, from_, to, typ=None):
-        # type: (unicode, unicode, unicode) -> unicode
+        # type: (str, str, str) -> str
         # ignore source path
         return self.get_target_uri(to, typ)
 
@@ -155,7 +166,7 @@ class LaTeXBuilder(Builder):
                               'will be written'))
             return
         # assign subdirs to titles
-        self.titles = []  # type: List[Tuple[unicode, unicode]]
+        self.titles = []  # type: List[Tuple[str, str]]
         for entry in preliminary_document_data:
             docname = entry[0]
             if docname not in self.env.all_docs:
@@ -167,6 +178,52 @@ class LaTeXBuilder(Builder):
                 docname = docname[:-5]
             self.titles.append((docname, entry[2]))
 
+    def init_context(self):
+        # type: () -> None
+        self.context = DEFAULT_SETTINGS.copy()
+
+        # Add special settings for latex_engine
+        self.context.update(ADDITIONAL_SETTINGS.get(self.config.latex_engine, {}))
+
+        # for xelatex+French, don't use polyglossia by default
+        if self.config.latex_engine == 'xelatex':
+            if self.config.language:
+                if self.config.language[:2] == 'fr':
+                    self.context['polyglossia'] = ''
+                    self.context['babel'] = r'\usepackage{babel}'
+
+        # Apply extension settings to context
+        self.context['packages'] = self.usepackages
+
+        # Apply user settings to context
+        self.context.update(self.config.latex_elements)
+        self.context['release'] = self.config.release
+        self.context['use_xindy'] = self.config.latex_use_xindy
+
+        if self.config.today:
+            self.context['date'] = self.config.today
+        else:
+            self.context['date'] = format_date(self.config.today_fmt or _('%b %d, %Y'),
+                                               language=self.config.language)
+
+        if self.config.latex_logo:
+            self.context['logofilename'] = path.basename(self.config.latex_logo)
+
+        # for compatibilities
+        self.context['indexname'] = _('Index')
+        if self.config.release:
+            # Show the release label only if release value exists
+            self.context.setdefault('releasename', _('Release'))
+
+    def init_babel(self):
+        # type: () -> None
+        self.babel = ExtBabel(self.config.language, not self.context['babel'])
+        if self.config.language and not self.babel.is_supported_language():
+            # emit warning if specified language is invalid
+            # (only emitting, nothing changed to processing)
+            logger.warning(__('no Babel option known for language %r'),
+                           self.config.language)
+
     def write_stylesheet(self):
         # type: () -> None
         highlighter = highlighting.PygmentsBridge('latex', self.config.pygments_style)
@@ -175,7 +232,7 @@ class LaTeXBuilder(Builder):
             f.write('\\NeedsTeXFormat{LaTeX2e}[1995/12/01]\n')
             f.write('\\ProvidesPackage{sphinxhighlight}'
                     '[2016/05/29 stylesheet for highlighting with pygments]\n\n')
-            f.write(highlighter.get_stylesheet())  # type: ignore
+            f.write(highlighter.get_stylesheet())
 
     def write(self, *ignored):
         # type: (Any) -> None
@@ -183,7 +240,7 @@ class LaTeXBuilder(Builder):
         docsettings = OptionParser(
             defaults=self.env.settings,
             components=(docwriter,),
-            read_config_files=True).get_default_values()
+            read_config_files=True).get_default_values()  # type: Any
 
         self.init_document_data()
         self.write_stylesheet()
@@ -195,33 +252,35 @@ class LaTeXBuilder(Builder):
                 toctree_only = entry[5]
             destination = SphinxFileOutput(destination_path=path.join(self.outdir, targetname),
                                            encoding='utf-8', overwrite_if_changed=True)
-            logger.info(__("processing %s..."), targetname, nonl=1)
-            toctrees = self.env.get_doctree(docname).traverse(addnodes.toctree)
-            if toctrees:
-                if toctrees[0].get('maxdepth') > 0:
-                    tocdepth = toctrees[0].get('maxdepth')
+            with progress_message(__("processing %s") % targetname):
+                toctrees = self.env.get_doctree(docname).traverse(addnodes.toctree)
+                if toctrees:
+                    if toctrees[0].get('maxdepth') > 0:
+                        tocdepth = toctrees[0].get('maxdepth')
+                    else:
+                        tocdepth = None
                 else:
                     tocdepth = None
-            else:
-                tocdepth = None
-            doctree = self.assemble_doctree(
-                docname, toctree_only,
-                appendices=((docclass != 'howto') and self.config.latex_appendices or []))
-            doctree['tocdepth'] = tocdepth
-            self.apply_transforms(doctree)
-            self.post_process_images(doctree)
-            logger.info(__("writing... "), nonl=1)
-            doctree.settings = docsettings
-            doctree.settings.author = author
-            doctree.settings.title = title
-            doctree.settings.contentsname = self.get_contentsname(docname)
-            doctree.settings.docname = docname
-            doctree.settings.docclass = docclass
-            docwriter.write(doctree, destination)
-            logger.info("done")
+                doctree = self.assemble_doctree(
+                    docname, toctree_only,
+                    appendices=((docclass != 'howto') and self.config.latex_appendices or []))
+                doctree['tocdepth'] = tocdepth
+                self.apply_transforms(doctree)
+                self.post_process_images(doctree)
+                self.update_doc_context(title, author)
+
+            with progress_message(__("writing")):
+                docsettings.author = author
+                docsettings.title = title
+                docsettings.contentsname = self.get_contentsname(docname)
+                docsettings.docname = docname
+                docsettings.docclass = docclass
+
+                doctree.settings = docsettings
+                docwriter.write(doctree, destination)
 
     def get_contentsname(self, indexfile):
-        # type: (unicode) -> unicode
+        # type: (str) -> str
         tree = self.env.get_doctree(indexfile)
         contentsname = None
         for toctree in tree.traverse(addnodes.toctree):
@@ -231,11 +290,15 @@ class LaTeXBuilder(Builder):
 
         return contentsname
 
+    def update_doc_context(self, title, author):
+        # type: (str, str) -> None
+        self.context['title'] = title
+        self.context['author'] = author
+
     def assemble_doctree(self, indexfile, toctree_only, appendices):
-        # type: (unicode, bool, List[unicode]) -> nodes.Node
-        from docutils import nodes  # NOQA
+        # type: (str, bool, List[str]) -> nodes.document
         self.docnames = set([indexfile] + appendices)
-        logger.info(darkgreen(indexfile) + " ", nonl=1)
+        logger.info(darkgreen(indexfile) + " ", nonl=True)
         tree = self.env.get_doctree(indexfile)
         tree['docname'] = indexfile
         if toctree_only:
@@ -243,8 +306,8 @@ class LaTeXBuilder(Builder):
             # fresh document
             new_tree = new_document('<latex output>')
             new_sect = nodes.section()
-            new_sect += nodes.title(u'<Set title in conf.py>',
-                                    u'<Set title in conf.py>')
+            new_sect += nodes.title('<Set title in conf.py>',
+                                    '<Set title in conf.py>')
             new_tree += new_sect
             for node in tree.traverse(addnodes.toctree):
                 new_sect += node
@@ -264,7 +327,7 @@ class LaTeXBuilder(Builder):
         for pendingnode in largetree.traverse(addnodes.pending_xref):
             docname = pendingnode['refdocname']
             sectname = pendingnode['refsectname']
-            newnodes = [nodes.emphasis(sectname, sectname)]
+            newnodes = [nodes.emphasis(sectname, sectname)]  # type: List[nodes.Node]
             for subdir, title in self.titles:
                 if docname.startswith(subdir):
                     newnodes.append(nodes.Text(_(' (in '), _(' (in ')))
@@ -284,14 +347,23 @@ class LaTeXBuilder(Builder):
                                     ShowUrlsTransform,
                                     LaTeXFootnoteTransform,
                                     LiteralBlockTransform,
-                                    DocumentTargetTransform])
+                                    DocumentTargetTransform,
+                                    IndexInSectionTitleTransform])
         transformer.apply_transforms()
 
     def finish(self):
         # type: () -> None
         self.copy_image_files()
+        self.write_message_catalog()
+        self.copy_support_files()
 
-        # copy TeX support files from texinputs
+        if self.config.latex_additional_files:
+            self.copy_latex_additional_files()
+
+    @progress_message(__('copying TeX support files'))
+    def copy_support_files(self):
+        # type: () -> None
+        """copy TeX support files from texinputs."""
         # configure usage of xindy (impacts Makefile and latexmkrc)
         # FIXME: convert this rather to a confval with suitable default
         #        according to language ? but would require extra documentation
@@ -322,21 +394,19 @@ class LaTeXBuilder(Builder):
             copy_asset_file(path.join(staticdirname, 'Makefile_t'),
                             self.outdir, context=context)
 
-        # copy additional files
-        if self.config.latex_additional_files:
-            logger.info(bold(__('copying additional files...')), nonl=1)
-            for filename in self.config.latex_additional_files:
-                logger.info(' ' + filename, nonl=1)
-                copy_asset_file(path.join(self.confdir, filename), self.outdir)
-            logger.info('')
-
         # the logo is handled differently
         if self.config.latex_logo:
             if not path.isfile(path.join(self.confdir, self.config.latex_logo)):
                 raise SphinxError(__('logo file %r does not exist') % self.config.latex_logo)
             else:
                 copy_asset_file(path.join(self.confdir, self.config.latex_logo), self.outdir)
-        logger.info(__('done'))
+
+    @progress_message(__('copying additional files'))
+    def copy_latex_additional_files(self):
+        # type: () -> None
+        for filename in self.config.latex_additional_files:
+            logger.info(' ' + filename, nonl=True)
+            copy_asset_file(path.join(self.confdir, filename), self.outdir)
 
     def copy_image_files(self):
         # type: () -> None
@@ -353,35 +423,34 @@ class LaTeXBuilder(Builder):
                     logger.warning(__('cannot copy image file %r: %s'),
                                    path.join(self.srcdir, src), err)
 
+    def write_message_catalog(self):
+        # type: () -> None
+        formats = self.config.numfig_format
+        context = {
+            'addtocaptions': r'\@iden',
+            'figurename': formats.get('figure', '').split('%s', 1),
+            'tablename': formats.get('table', '').split('%s', 1),
+            'literalblockname': formats.get('code-block', '').split('%s', 1)
+        }
+
+        if self.context['babel'] or self.context['polyglossia']:
+            context['addtocaptions'] = r'\addto\captions%s' % self.babel.get_language()
+
+        filename = path.join(package_dir, 'templates', 'latex', 'sphinxmessages.sty_t')
+        copy_asset_file(filename, self.outdir, context=context, renderer=LaTeXRenderer())
+
 
 def validate_config_values(app, config):
     # type: (Sphinx, Config) -> None
-    for document in config.latex_documents:
-        try:
-            text_type(document[2])
-        except UnicodeDecodeError:
-            raise ConfigError(
-                __('Invalid latex_documents.title found (might contain non-ASCII chars. '
-                   'Please use u"..." notation instead): %r') % (document,)
-            )
-
-        try:
-            text_type(document[3])
-        except UnicodeDecodeError:
-            raise ConfigError(
-                __('Invalid latex_documents.author found (might contain non-ASCII chars. '
-                   'Please use u"..." notation instead): %r') % (document,)
-            )
-
     for key in list(config.latex_elements):
         if key not in DEFAULT_SETTINGS:
             msg = __("Unknown configure key: latex_elements[%r]. ignored.")
-            logger.warning(msg % key)
+            logger.warning(msg % (key,))
             config.latex_elements.pop(key)
 
 
 def default_latex_engine(config):
-    # type: (Config) -> unicode
+    # type: (Config) -> str
     """ Better default latex_engine settings for specific languages. """
     if config.language == 'ja':
         return 'platex'
@@ -390,7 +459,7 @@ def default_latex_engine(config):
 
 
 def default_latex_docclass(config):
-    # type: (Config) -> Dict[unicode, unicode]
+    # type: (Config) -> Dict[str, str]
     """ Better default latex_docclass settings for specific languages. """
     if config.language == 'ja':
         return {'manual': 'jsbook',
@@ -405,8 +474,18 @@ def default_latex_use_xindy(config):
     return config.latex_engine in {'xelatex', 'lualatex'}
 
 
+def default_latex_documents(config):
+    # type: (Config) -> List[Tuple[str, str, str, str, str]]
+    """ Better default latex_documents settings. """
+    return [(config.master_doc,
+             make_filename_from_project(config.project) + '.tex',
+             texescape.escape_abbr(texescape.escape(config.project)),
+             texescape.escape_abbr(texescape.escape(config.author)),
+             'manual')]
+
+
 def setup(app):
-    # type: (Sphinx) -> Dict[unicode, Any]
+    # type: (Sphinx) -> Dict[str, Any]
     app.add_builder(LaTeXBuilder)
     app.add_post_transform(CitationReferenceTransform)
     app.add_post_transform(MathReferenceTransform)
@@ -415,11 +494,8 @@ def setup(app):
 
     app.add_config_value('latex_engine', default_latex_engine, None,
                          ENUM('pdflatex', 'xelatex', 'lualatex', 'platex'))
-    app.add_config_value('latex_documents',
-                         lambda self: [(self.master_doc, make_filename(self.project) + '.tex',
-                                        self.project, '', 'manual')],
-                         None)
-    app.add_config_value('latex_logo', None, None, string_classes)
+    app.add_config_value('latex_documents', default_latex_documents, None)
+    app.add_config_value('latex_logo', None, None, [str])
     app.add_config_value('latex_appendices', [], None)
     app.add_config_value('latex_use_latex_multicolumn', False, None)
     app.add_config_value('latex_use_xindy', default_latex_use_xindy, None)
