@@ -1,11 +1,10 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.builders.texinfo
     ~~~~~~~~~~~~~~~~~~~~~~~
 
     Texinfo builder.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -17,79 +16,29 @@ from docutils.frontend import OptionParser
 from docutils.io import FileOutput
 
 from sphinx import addnodes
+from sphinx import package_dir
 from sphinx.builders import Builder
 from sphinx.environment import NoUri
 from sphinx.environment.adapters.asset import ImageAdapter
 from sphinx.locale import _, __
 from sphinx.util import logging
-from sphinx.util import status_iterator
-from sphinx.util.console import bold, darkgreen  # type: ignore
+from sphinx.util import progress_message, status_iterator
+from sphinx.util.console import darkgreen  # type: ignore
 from sphinx.util.docutils import new_document
 from sphinx.util.fileutil import copy_asset_file
 from sphinx.util.nodes import inline_all_toctrees
-from sphinx.util.osutil import SEP, make_filename
+from sphinx.util.osutil import SEP, ensuredir, make_filename_from_project
 from sphinx.writers.texinfo import TexinfoWriter, TexinfoTranslator
 
 if False:
     # For type annotation
     from sphinx.application import Sphinx  # NOQA
+    from sphinx.config import Config  # NOQA
     from typing import Any, Dict, Iterable, List, Tuple, Union  # NOQA
 
 
 logger = logging.getLogger(__name__)
-
-TEXINFO_MAKEFILE = '''\
-# Makefile for Sphinx Texinfo output
-
-infodir ?= /usr/share/info
-
-MAKEINFO = makeinfo --no-split
-MAKEINFO_html = makeinfo --no-split --html
-MAKEINFO_plaintext = makeinfo --no-split --plaintext
-TEXI2PDF = texi2pdf --batch --expand
-INSTALL_INFO = install-info
-
-ALLDOCS = $(basename $(wildcard *.texi))
-
-all: info
-info: $(addsuffix .info,$(ALLDOCS))
-plaintext: $(addsuffix .txt,$(ALLDOCS))
-html: $(addsuffix .html,$(ALLDOCS))
-pdf: $(addsuffix .pdf,$(ALLDOCS))
-
-install-info: info
-\tfor f in *.info; do \\
-\t  cp -t $(infodir) "$$f" && \\
-\t  $(INSTALL_INFO) --info-dir=$(infodir) "$$f" ; \\
-\tdone
-
-uninstall-info: info
-\tfor f in *.info; do \\
-\t  rm -f "$(infodir)/$$f"  ; \\
-\t  $(INSTALL_INFO) --delete --info-dir=$(infodir) "$$f" ; \\
-\tdone
-
-%.info: %.texi
-\t$(MAKEINFO) -o '$@' '$<'
-
-%.txt: %.texi
-\t$(MAKEINFO_plaintext) -o '$@' '$<'
-
-%.html: %.texi
-\t$(MAKEINFO_html) -o '$@' '$<'
-
-%.pdf: %.texi
-\t-$(TEXI2PDF) '$<'
-\t-$(TEXI2PDF) '$<'
-\t-$(TEXI2PDF) '$<'
-
-clean:
-\trm -f *.info *.pdf *.txt *.html
-\trm -f *.log *.ind *.aux *.toc *.syn *.idx *.out *.ilg *.pla *.ky *.pg
-\trm -f *.vr *.tp *.fn *.fns *.def *.defs *.cp *.cps *.ge *.ges *.mo
-
-.PHONY: all info plaintext html pdf install-info uninstall-info clean
-'''
+template_dir = os.path.join(package_dir, 'templates', 'texinfo')
 
 
 class TexinfoBuilder(Builder):
@@ -110,22 +59,22 @@ class TexinfoBuilder(Builder):
 
     def init(self):
         # type: () -> None
-        self.docnames = []       # type: Iterable[unicode]
-        self.document_data = []  # type: List[Tuple[unicode, unicode, unicode, unicode, unicode, unicode, unicode, bool]]  # NOQA
+        self.docnames = []       # type: Iterable[str]
+        self.document_data = []  # type: List[Tuple[str, str, str, str, str, str, str, bool]]
 
     def get_outdated_docs(self):
-        # type: () -> Union[unicode, List[unicode]]
+        # type: () -> Union[str, List[str]]
         return 'all documents'  # for now
 
     def get_target_uri(self, docname, typ=None):
-        # type: (unicode, unicode) -> unicode
+        # type: (str, str) -> str
         if docname not in self.docnames:
             raise NoUri
         else:
             return '%' + docname
 
     def get_relative_uri(self, from_, to, typ=None):
-        # type: (unicode, unicode, unicode) -> unicode
+        # type: (str, str, str) -> str
         # ignore source path
         return self.get_target_uri(to, typ)
 
@@ -137,7 +86,7 @@ class TexinfoBuilder(Builder):
                               'will be written'))
             return
         # assign subdirs to titles
-        self.titles = []  # type: List[Tuple[unicode, unicode]]
+        self.titles = []  # type: List[Tuple[str, str]]
         for entry in preliminary_document_data:
             docname = entry[0]
             if docname not in self.env.all_docs:
@@ -155,7 +104,7 @@ class TexinfoBuilder(Builder):
         for entry in self.document_data:
             docname, targetname, title, author = entry[:4]
             targetname += '.texi'
-            direntry = description = category = ''  # type: unicode
+            direntry = description = category = ''
             if len(entry) > 6:
                 direntry, description, category = entry[4:7]
             toctree_only = False
@@ -164,33 +113,33 @@ class TexinfoBuilder(Builder):
             destination = FileOutput(
                 destination_path=path.join(self.outdir, targetname),
                 encoding='utf-8')
-            logger.info(__("processing %s..."), targetname, nonl=1)
-            doctree = self.assemble_doctree(
-                docname, toctree_only,
-                appendices=(self.config.texinfo_appendices or []))
-            logger.info(__("writing... "), nonl=1)
-            self.post_process_images(doctree)
-            docwriter = TexinfoWriter(self)
-            settings = OptionParser(
-                defaults=self.env.settings,
-                components=(docwriter,),
-                read_config_files=True).get_default_values()
-            settings.author = author
-            settings.title = title
-            settings.texinfo_filename = targetname[:-5] + '.info'
-            settings.texinfo_elements = self.config.texinfo_elements
-            settings.texinfo_dir_entry = direntry or ''
-            settings.texinfo_dir_category = category or ''
-            settings.texinfo_dir_description = description or ''
-            settings.docname = docname
-            doctree.settings = settings
-            docwriter.write(doctree, destination)
-            logger.info(__("done"))
+            with progress_message(__("processing %s") % targetname):
+                appendices = self.config.texinfo_appendices or []
+                doctree = self.assemble_doctree(docname, toctree_only, appendices=appendices)
+
+            with progress_message(__("writing")):
+                self.post_process_images(doctree)
+                docwriter = TexinfoWriter(self)
+                settings = OptionParser(
+                    defaults=self.env.settings,
+                    components=(docwriter,),
+                    read_config_files=True).get_default_values()  # type: Any
+                settings.author = author
+                settings.title = title
+                settings.texinfo_filename = targetname[:-5] + '.info'
+                settings.texinfo_elements = self.config.texinfo_elements
+                settings.texinfo_dir_entry = direntry or ''
+                settings.texinfo_dir_category = category or ''
+                settings.texinfo_dir_description = description or ''
+                settings.docname = docname
+                doctree.settings = settings
+                docwriter.write(doctree, destination)
+                self.copy_image_files(targetname[:-5])
 
     def assemble_doctree(self, indexfile, toctree_only, appendices):
-        # type: (unicode, bool, List[unicode]) -> nodes.Node
+        # type: (str, bool, List[str]) -> nodes.document
         self.docnames = set([indexfile] + appendices)
-        logger.info(darkgreen(indexfile) + " ", nonl=1)
+        logger.info(darkgreen(indexfile) + " ", nonl=True)
         tree = self.env.get_doctree(indexfile)
         tree['docname'] = indexfile
         if toctree_only:
@@ -198,8 +147,8 @@ class TexinfoBuilder(Builder):
             # fresh document
             new_tree = new_document('<texinfo output>')
             new_sect = nodes.section()
-            new_sect += nodes.title(u'<Set title in conf.py>',
-                                    u'<Set title in conf.py>')
+            new_sect += nodes.title('<Set title in conf.py>',
+                                    '<Set title in conf.py>')
             new_tree += new_sect
             for node in tree.traverse(addnodes.toctree):
                 new_sect += node
@@ -218,7 +167,7 @@ class TexinfoBuilder(Builder):
         for pendingnode in largetree.traverse(addnodes.pending_xref):
             docname = pendingnode['refdocname']
             sectname = pendingnode['refsectname']
-            newnodes = [nodes.emphasis(sectname, sectname)]
+            newnodes = [nodes.emphasis(sectname, sectname)]  # type: List[nodes.Node]
             for subdir, title in self.titles:
                 if docname.startswith(subdir):
                     newnodes.append(nodes.Text(_(' (in '), _(' (in ')))
@@ -232,21 +181,10 @@ class TexinfoBuilder(Builder):
 
     def finish(self):
         # type: () -> None
-        self.copy_image_files()
+        self.copy_support_files()
 
-        logger.info(bold(__('copying Texinfo support files... ')), nonl=True)
-        # copy Makefile
-        fn = path.join(self.outdir, 'Makefile')
-        logger.info(fn, nonl=1)
-        try:
-            with open(fn, 'w') as mkfile:
-                mkfile.write(TEXINFO_MAKEFILE)
-        except (IOError, OSError) as err:
-            logger.warning(__("error writing file %s: %s"), fn, err)
-        logger.info(__(' done'))
-
-    def copy_image_files(self):
-        # type: () -> None
+    def copy_image_files(self, targetname):
+        # type: (str) -> None
         if self.images:
             stringify_func = ImageAdapter(self.app.env).get_original_image_uri
             for src in status_iterator(self.images, __('copying images... '), "brown",
@@ -254,24 +192,36 @@ class TexinfoBuilder(Builder):
                                        stringify_func=stringify_func):
                 dest = self.images[src]
                 try:
-                    copy_asset_file(path.join(self.srcdir, src),
-                                    path.join(self.outdir, dest))
+                    imagedir = path.join(self.outdir, targetname + '-figures')
+                    ensuredir(imagedir)
+                    copy_asset_file(path.join(self.srcdir, dest), imagedir)
                 except Exception as err:
                     logger.warning(__('cannot copy image file %r: %s'),
                                    path.join(self.srcdir, src), err)
 
+    def copy_support_files(self):
+        # type: () -> None
+        try:
+            with progress_message(__('copying Texinfo support files')):
+                logger.info('Makefile ', nonl=True)
+                copy_asset_file(os.path.join(template_dir, 'Makefile'), self.outdir)
+        except OSError as err:
+            logger.warning(__("error writing file Makefile: %s"), err)
+
+
+def default_texinfo_documents(config):
+    # type: (Config) -> List[Tuple[str, str, str, str, str, str, str]]
+    """ Better default texinfo_documents settings. """
+    filename = make_filename_from_project(config.project)
+    return [(config.master_doc, filename, config.project, config.author, filename,
+             'One line description of project', 'Miscellaneous')]
+
 
 def setup(app):
-    # type: (Sphinx) -> Dict[unicode, Any]
+    # type: (Sphinx) -> Dict[str, Any]
     app.add_builder(TexinfoBuilder)
 
-    app.add_config_value('texinfo_documents',
-                         lambda self: [(self.master_doc, make_filename(self.project).lower(),
-                                        self.project, '', make_filename(self.project),
-                                        'The %s reference manual.' %
-                                        make_filename(self.project),
-                                        'Python')],
-                         None)
+    app.add_config_value('texinfo_documents', default_texinfo_documents, None)
     app.add_config_value('texinfo_appendices', [], None)
     app.add_config_value('texinfo_elements', {}, None)
     app.add_config_value('texinfo_domain_indices', True, None, [list])
