@@ -1,15 +1,13 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.builders.xml
     ~~~~~~~~~~~~~~~~~~~
 
     Docutils-native XML and pseudo-XML builders.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-import codecs
 from os import path
 
 from docutils import nodes
@@ -17,13 +15,15 @@ from docutils.io import StringOutput
 from docutils.writers.docutils_xml import XMLTranslator
 
 from sphinx.builders import Builder
+from sphinx.locale import __
 from sphinx.util import logging
 from sphinx.util.osutil import ensuredir, os_path
 from sphinx.writers.xml import XMLWriter, PseudoXMLWriter
 
 if False:
     # For type annotation
-    from typing import Any, Dict, Iterator, Set  # NOQA
+    from typing import Any, Dict, Iterator, Set, Type  # NOQA
+    from docutils.writers.xml import BaseXMLWriter  # NOQA
     from sphinx.application import Sphinx  # NOQA
 
 logger = logging.getLogger(__name__)
@@ -35,10 +35,12 @@ class XMLBuilder(Builder):
     """
     name = 'xml'
     format = 'xml'
+    epilog = __('The XML files are in %(outdir)s.')
+
     out_suffix = '.xml'
     allow_parallel = True
 
-    _writer_class = XMLWriter
+    _writer_class = XMLWriter  # type: Type[BaseXMLWriter]
     default_translator_class = XMLTranslator
 
     def init(self):
@@ -46,13 +48,12 @@ class XMLBuilder(Builder):
         pass
 
     def get_outdated_docs(self):
-        # type: () -> Iterator[unicode]
+        # type: () -> Iterator[str]
         for docname in self.env.found_docs:
             if docname not in self.env.all_docs:
                 yield docname
                 continue
-            targetname = self.env.doc2path(docname, self.outdir,
-                                           self.out_suffix)
+            targetname = path.join(self.outdir, docname + self.out_suffix)
             try:
                 targetmtime = path.getmtime(targetname)
             except Exception:
@@ -61,20 +62,20 @@ class XMLBuilder(Builder):
                 srcmtime = path.getmtime(self.env.doc2path(docname))
                 if srcmtime > targetmtime:
                     yield docname
-            except EnvironmentError:
+            except OSError:
                 # source doesn't exist anymore
                 pass
 
     def get_target_uri(self, docname, typ=None):
-        # type: (unicode, unicode) -> unicode
+        # type: (str, str) -> str
         return docname
 
     def prepare_writing(self, docnames):
-        # type: (Set[unicode]) -> None
+        # type: (Set[str]) -> None
         self.writer = self._writer_class(self)
 
     def write_doc(self, docname, doctree):
-        # type: (unicode, nodes.Node) -> None
+        # type: (str, nodes.Node) -> None
         # work around multiple string % tuple issues in docutils;
         # replace tuples in attribute values with lists
         doctree = doctree.deepcopy()
@@ -92,10 +93,10 @@ class XMLBuilder(Builder):
         outfilename = path.join(self.outdir, os_path(docname) + self.out_suffix)
         ensuredir(path.dirname(outfilename))
         try:
-            with codecs.open(outfilename, 'w', 'utf-8') as f:  # type: ignore
+            with open(outfilename, 'w', encoding='utf-8') as f:
                 f.write(self.writer.output)
-        except (IOError, OSError) as err:
-            logger.warning("error writing file %s: %s", outfilename, err)
+        except OSError as err:
+            logger.warning(__("error writing file %s: %s"), outfilename, err)
 
     def finish(self):
         # type: () -> None
@@ -108,13 +109,15 @@ class PseudoXMLBuilder(XMLBuilder):
     """
     name = 'pseudoxml'
     format = 'pseudoxml'
+    epilog = __('The pseudo-XML files are in %(outdir)s.')
+
     out_suffix = '.pseudoxml'
 
     _writer_class = PseudoXMLWriter
 
 
 def setup(app):
-    # type: (Sphinx) -> Dict[unicode, Any]
+    # type: (Sphinx) -> Dict[str, Any]
     app.add_builder(XMLBuilder)
     app.add_builder(PseudoXMLBuilder)
 
