@@ -1,15 +1,12 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.cmd.quickstart
     ~~~~~~~~~~~~~~~~~~~~~
 
     Quickly setup documentation source to work with Sphinx.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
-from __future__ import absolute_import
-from __future__ import print_function
 
 import argparse
 import locale
@@ -17,8 +14,8 @@ import os
 import re
 import sys
 import time
+import warnings
 from collections import OrderedDict
-from io import open
 from os import path
 
 # try to import readline, unix specific enhancement
@@ -34,25 +31,22 @@ except ImportError:
     USE_LIBEDIT = False
 
 from docutils.utils import column_width
-from six import PY2, PY3, text_type, binary_type
-from six.moves import input
-from six.moves.urllib.parse import quote as urlquote
 
 import sphinx.locale
 from sphinx import __display_version__, package_dir
+from sphinx.deprecation import RemovedInSphinx40Warning
 from sphinx.locale import __
-from sphinx.util import texescape
 from sphinx.util.console import (  # type: ignore
     colorize, bold, red, turquoise, nocolor, color_terminal
 )
-from sphinx.util.osutil import ensuredir, make_filename
+from sphinx.util.osutil import ensuredir
 from sphinx.util.template import SphinxRenderer
 
 if False:
     # For type annotation
     from typing import Any, Callable, Dict, List, Pattern, Union  # NOQA
 
-TERM_ENCODING = getattr(sys.stdin, 'encoding', None)
+TERM_ENCODING = getattr(sys.stdin, 'encoding', None)  # RemovedInSphinx40Warning
 
 EXTENSIONS = OrderedDict([
     ('autodoc', __('automatically insert docstrings from modules')),
@@ -91,7 +85,7 @@ else:
 
 # function to get input from terminal -- overridden by the test suite
 def term_input(prompt):
-    # type: (unicode) -> unicode
+    # type: (str) -> str
     if sys.platform == 'win32':
         # Important: On windows, readline is not enabled by default.  In these
         #            environment, escape sequences have been broken.  To avoid the
@@ -107,29 +101,29 @@ class ValidationError(Exception):
 
 
 def is_path(x):
-    # type: (unicode) -> unicode
+    # type: (str) -> str
     x = path.expanduser(x)
-    if path.exists(x) and not path.isdir(x):
+    if not path.isdir(x):
         raise ValidationError(__("Please enter a valid path name."))
     return x
 
 
 def allow_empty(x):
-    # type: (unicode) -> unicode
+    # type: (str) -> str
     return x
 
 
 def nonempty(x):
-    # type: (unicode) -> unicode
+    # type: (str) -> str
     if not x:
         raise ValidationError(__("Please enter some text."))
     return x
 
 
 def choice(*l):
-    # type: (unicode) -> Callable[[unicode], unicode]
+    # type: (str) -> Callable[[str], str]
     def val(x):
-        # type: (unicode) -> unicode
+        # type: (str) -> str
         if x not in l:
             raise ValidationError(__('Please enter one of %s.') % ', '.join(l))
         return x
@@ -137,14 +131,14 @@ def choice(*l):
 
 
 def boolean(x):
-    # type: (unicode) -> bool
+    # type: (str) -> bool
     if x.upper() not in ('Y', 'YES', 'N', 'NO'):
         raise ValidationError(__("Please enter either 'y' or 'n'."))
     return x.upper() in ('Y', 'YES')
 
 
 def suffix(x):
-    # type: (unicode) -> unicode
+    # type: (str) -> str
     if not (x[0:1] == '.' and len(x) > 1):
         raise ValidationError(__("Please enter a file suffix, "
                                  "e.g. '.rst' or '.txt'."))
@@ -152,13 +146,16 @@ def suffix(x):
 
 
 def ok(x):
-    # type: (unicode) -> unicode
+    # type: (str) -> str
     return x
 
 
 def term_decode(text):
-    # type: (Union[bytes,unicode]) -> unicode
-    if isinstance(text, text_type):
+    # type: (Union[bytes,str]) -> str
+    warnings.warn('term_decode() is deprecated.',
+                  RemovedInSphinx40Warning, stacklevel=2)
+
+    if isinstance(text, str):
         return text
 
     # Use the known encoding, if possible
@@ -173,32 +170,18 @@ def term_decode(text):
                        'and terminal encoding unknown -- assuming '
                        'UTF-8 or Latin-1.')))
     try:
-        return text.decode('utf-8')
+        return text.decode()
     except UnicodeDecodeError:
         return text.decode('latin1')
 
 
 def do_prompt(text, default=None, validator=nonempty):
-    # type: (unicode, unicode, Callable[[unicode], Any]) -> Union[unicode, bool]
+    # type: (str, str, Callable[[str], Any]) -> Union[str, bool]
     while True:
         if default is not None:
-            prompt = PROMPT_PREFIX + '%s [%s]: ' % (text, default)  # type: unicode
+            prompt = PROMPT_PREFIX + '%s [%s]: ' % (text, default)
         else:
             prompt = PROMPT_PREFIX + text + ': '
-        if PY2:
-            # for Python 2.x, try to get a Unicode string out of it
-            if prompt.encode('ascii', 'replace').decode('ascii', 'replace') \
-                    != prompt:
-                if TERM_ENCODING:
-                    prompt = prompt.encode(TERM_ENCODING)
-                else:
-                    print(turquoise(__('* Note: non-ASCII default value provided '
-                                       'and terminal encoding unknown -- assuming '
-                                       'UTF-8 or Latin-1.')))
-                    try:
-                        prompt = prompt.encode('utf-8')
-                    except UnicodeEncodeError:
-                        prompt = prompt.encode('latin1')
         if USE_LIBEDIT:
             # Note: libedit has a problem for combination of ``input()`` and escape
             # sequence (see #5335).  To avoid the problem, all prompts are not colored
@@ -209,7 +192,6 @@ def do_prompt(text, default=None, validator=nonempty):
         x = term_input(prompt).strip()
         if default and not x:
             x = default
-        x = term_decode(x)
         try:
             x = validator(x)
         except ValidationError as err:
@@ -220,27 +202,26 @@ def do_prompt(text, default=None, validator=nonempty):
 
 
 def convert_python_source(source, rex=re.compile(r"[uU]('.*?')")):
-    # type: (unicode, Pattern) -> unicode
+    # type: (str, Pattern) -> str
     # remove Unicode literal prefixes
-    if PY3:
-        return rex.sub('\\1', source)
-    else:
-        return source
+    warnings.warn('convert_python_source() is deprecated.',
+                  RemovedInSphinx40Warning)
+    return rex.sub('\\1', source)
 
 
 class QuickstartRenderer(SphinxRenderer):
     def __init__(self, templatedir):
-        # type: (unicode) -> None
+        # type: (str) -> None
         self.templatedir = templatedir or ''
-        super(QuickstartRenderer, self).__init__()
+        super().__init__()
 
     def render(self, template_name, context):
-        # type: (unicode, Dict) -> unicode
+        # type: (str, Dict) -> str
         user_template = path.join(self.templatedir, path.basename(template_name))
         if self.templatedir and path.exists(user_template):
             return self.render_from_file(user_template, context)
         else:
-            return super(QuickstartRenderer, self).render(template_name, context)
+            return super().render(template_name, context)
 
 
 def ask_user(d):
@@ -368,8 +349,7 @@ document is a custom template, you can also set this to another filename.'''))
                 d['extensions'].append('sphinx.ext.%s' % name)
 
         # Handle conflicting options
-        if set(['sphinx.ext.imgmath', 'sphinx.ext.mathjax']).issubset(
-                d['extensions']):
+        if {'sphinx.ext.imgmath', 'sphinx.ext.mathjax'}.issubset(d['extensions']):
             print(__('Note: imgmath and mathjax cannot be enabled at the same '
                      'time. imgmath has been deselected.'))
             d['extensions'].remove('sphinx.ext.imgmath')
@@ -388,40 +368,21 @@ directly.'''))
 
 
 def generate(d, overwrite=True, silent=False, templatedir=None):
-    # type: (Dict, bool, bool, unicode) -> None
+    # type: (Dict, bool, bool, str) -> None
     """Generate project based on values in *d*."""
     template = QuickstartRenderer(templatedir=templatedir)
 
-    texescape.init()
-
     if 'mastertoctree' not in d:
         d['mastertoctree'] = ''
     if 'mastertocmaxdepth' not in d:
         d['mastertocmaxdepth'] = 2
 
-    d['PY3'] = PY3
-    d['project_fn'] = make_filename(d['project'])
-    d['project_url'] = urlquote(d['project'].encode('idna'))
-    d['project_manpage'] = d['project_fn'].lower()
     d['now'] = time.asctime()
     d['project_underline'] = column_width(d['project']) * '='
     d.setdefault('extensions', [])
     d['copyright'] = time.strftime('%Y') + ', ' + d['author']
-    d['author_texescaped'] = text_type(d['author']).\
-        translate(texescape.tex_escape_map)
-    d['project_doc'] = d['project'] + ' Documentation'
-    d['project_doc_texescaped'] = text_type(d['project'] + ' Documentation').\
-        translate(texescape.tex_escape_map)
 
-    # escape backslashes and single quotes in strings that are put into
-    # a Python string literal
-    for key in ('project', 'project_doc', 'project_doc_texescaped',
-                'author', 'author_texescaped', 'copyright',
-                'version', 'release', 'master'):
-        d[key + '_str'] = d[key].replace('\\', '\\\\').replace("'", "\\'")
-
-    if not path.isdir(d['path']):
-        ensuredir(d['path'])
+    ensuredir(d['path'])
 
     srcdir = d['sep'] and path.join(d['path'], 'source') or d['path']
 
@@ -441,7 +402,7 @@ def generate(d, overwrite=True, silent=False, templatedir=None):
     ensuredir(path.join(srcdir, d['dot'] + 'static'))
 
     def write_file(fpath, content, newline=None):
-        # type: (unicode, unicode, unicode) -> None
+        # type: (str, str, str) -> None
         if overwrite or not path.isfile(fpath):
             if 'quiet' not in d:
                 print(__('Creating file %s.') % fpath)
@@ -455,7 +416,7 @@ def generate(d, overwrite=True, silent=False, templatedir=None):
     if not conf_path or not path.isfile(conf_path):
         conf_path = os.path.join(package_dir, 'templates', 'quickstart', 'conf.py_t')
     with open(conf_path) as f:
-        conf_text = convert_python_source(f.read())
+        conf_text = f.read()
 
     write_file(path.join(srcdir, 'conf.py'), template.render_string(conf_text, d))
 
@@ -474,13 +435,13 @@ def generate(d, overwrite=True, silent=False, templatedir=None):
         d['rbuilddir'] = d['sep'] and 'build' or d['dot'] + 'build'
         # use binary mode, to avoid writing \r\n on Windows
         write_file(path.join(d['path'], 'Makefile'),
-                   template.render(makefile_template, d), u'\n')
+                   template.render(makefile_template, d), '\n')
 
     if d['batchfile'] is True:
         d['rsrcdir'] = d['sep'] and 'source' or '.'
         d['rbuilddir'] = d['sep'] and 'build' or d['dot'] + 'build'
         write_file(path.join(d['path'], 'make.bat'),
-                   template.render(batchfile_template, d), u'\r\n')
+                   template.render(batchfile_template, d), '\r\n')
 
     if silent:
         return
@@ -507,7 +468,7 @@ def valid_dir(d):
     if not path.isdir(dir):
         return False
 
-    if set(['Makefile', 'make.bat']) & set(os.listdir(dir)):  # type: ignore
+    if {'Makefile', 'make.bat'} & set(os.listdir(dir)):
         return False
 
     if d['sep']:
@@ -523,7 +484,7 @@ def valid_dir(d):
         d['dot'] + 'templates',
         d['master'] + d['suffix'],
     ]
-    if set(reserved_names) & set(os.listdir(dir)):  # type: ignore
+    if set(reserved_names) & set(os.listdir(dir)):
         return False
 
     return True
@@ -543,7 +504,7 @@ Makefile to be used with sphinx-build.
 """))
 
     parser.add_argument('-q', '--quiet', action='store_true', dest='quiet',
-                        default=False,
+                        default=None,
                         help=__('quiet mode'))
     parser.add_argument('--version', action='version', dest='show_version',
                         version='%%(prog)s %s' % __display_version__)
@@ -552,9 +513,9 @@ Makefile to be used with sphinx-build.
                         help=__('output path'))
 
     group = parser.add_argument_group(__('Structure options'))
-    group.add_argument('--sep', action='store_true',
+    group.add_argument('--sep', action='store_true', default=None,
                        help=__('if specified, separate source and build dirs'))
-    group.add_argument('--dot', metavar='DOT',
+    group.add_argument('--dot', metavar='DOT', default='_',
                        help=__('replacement for dot in _templates etc.'))
 
     group = parser.add_argument_group(__('Project basic options'))
@@ -568,9 +529,9 @@ Makefile to be used with sphinx-build.
                        help=__('release of project'))
     group.add_argument('-l', '--language', metavar='LANGUAGE', dest='language',
                        help=__('document language'))
-    group.add_argument('--suffix', metavar='SUFFIX',
+    group.add_argument('--suffix', metavar='SUFFIX', default='.rst',
                        help=__('source file suffix'))
-    group.add_argument('--master', metavar='MASTER',
+    group.add_argument('--master', metavar='MASTER', default='index',
                        help=__('master document name'))
     group.add_argument('--epub', action='store_true', default=False,
                        help=__('use epub'))
@@ -584,11 +545,11 @@ Makefile to be used with sphinx-build.
                        action='append', help=__('enable arbitrary extensions'))
 
     group = parser.add_argument_group(__('Makefile and Batchfile creation'))
-    group.add_argument('--makefile', action='store_true', dest='makefile',
+    group.add_argument('--makefile', action='store_true', dest='makefile', default=True,
                        help=__('create makefile'))
     group.add_argument('--no-makefile', action='store_false', dest='makefile',
                        help=__('do not create makefile'))
-    group.add_argument('--batchfile', action='store_true', dest='batchfile',
+    group.add_argument('--batchfile', action='store_true', dest='batchfile', default=True,
                        help=__('create batchfile'))
     group.add_argument('--no-batchfile', action='store_false',
                        dest='batchfile',
@@ -613,7 +574,7 @@ Makefile to be used with sphinx-build.
 
 def main(argv=sys.argv[1:]):
     # type: (List[str]) -> int
-    locale.setlocale(locale.LC_ALL, '')
+    sphinx.locale.setlocale(locale.LC_ALL, '')
     sphinx.locale.init_console(os.path.join(package_dir, 'locale'), 'sphinx')
 
     if not color_terminal():
@@ -628,16 +589,23 @@ def main(argv=sys.argv[1:]):
 
     d = vars(args)
     # delete None or False value
-    d = dict((k, v) for k, v in d.items() if not (v is None or v is False))
+    d = {k: v for k, v in d.items() if v is not None}
+
+    # handle use of CSV-style extension values
+    d.setdefault('extensions', [])
+    for ext in d['extensions'][:]:
+        if ',' in ext:
+            d['extensions'].remove(ext)
+            d['extensions'].extend(ext.split(','))
 
     try:
         if 'quiet' in d:
-            if not set(['project', 'author']).issubset(d):
+            if not {'project', 'author'}.issubset(d):
                 print(__('''"quiet" is specified, but any of "project" or \
 "author" is not specified.'''))
                 return 1
 
-        if set(['quiet', 'project', 'author']).issubset(d):
+        if {'quiet', 'project', 'author'}.issubset(d):
             # quiet mode with all required params satisfied, use default
             d.setdefault('version', '')
             d.setdefault('release', d['version'])
@@ -659,18 +627,6 @@ def main(argv=sys.argv[1:]):
         print('[Interrupted.]')
         return 130  # 128 + SIGINT
 
-    # decode values in d if value is a Python string literal
-    for key, value in d.items():
-        if isinstance(value, binary_type):
-            d[key] = term_decode(value)
-
-    # handle use of CSV-style extension values
-    d.setdefault('extensions', [])
-    for ext in d['extensions'][:]:
-        if ',' in ext:
-            d['extensions'].remove(ext)
-            d['extensions'].extend(ext.split(','))
-
     for variable in d.get('variables', []):
         try:
             name, value = variable.split('=')
