@@ -1,18 +1,15 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.domains.rst
     ~~~~~~~~~~~~~~~~~~
 
     The reStructuredText domain.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import re
 
-from six import iteritems
-
 from sphinx import addnodes
 from sphinx.directives import ObjectDescription
 from sphinx.domains import Domain, ObjType
@@ -38,7 +35,7 @@ class ReSTMarkup(ObjectDescription):
     """
 
     def add_target_and_index(self, name, sig, signode):
-        # type: (unicode, unicode, addnodes.desc_signature) -> None
+        # type: (str, str, addnodes.desc_signature) -> None
         targetname = self.objtype + '-' + name
         if targetname not in self.state.document.ids:
             signode['names'].append(targetname)
@@ -60,7 +57,7 @@ class ReSTMarkup(ObjectDescription):
                                               targetname, '', None))
 
     def get_index_text(self, objectname, name):
-        # type: (unicode, unicode) -> unicode
+        # type: (str, str) -> str
         if self.objtype == 'directive':
             return _('%s (directive)') % name
         elif self.objtype == 'role':
@@ -69,7 +66,7 @@ class ReSTMarkup(ObjectDescription):
 
 
 def parse_directive(d):
-    # type: (unicode) -> Tuple[unicode, unicode]
+    # type: (str) -> Tuple[str, str]
     """Parse a directive signature.
 
     Returns (directive, arguments) string tuple.  If no arguments are given,
@@ -91,7 +88,7 @@ class ReSTDirective(ReSTMarkup):
     Description of a reST directive.
     """
     def handle_signature(self, sig, signode):
-        # type: (unicode, addnodes.desc_signature) -> unicode
+        # type: (str, addnodes.desc_signature) -> str
         name, args = parse_directive(sig)
         desc_name = '.. %s::' % name
         signode += addnodes.desc_name(desc_name, desc_name)
@@ -105,7 +102,7 @@ class ReSTRole(ReSTMarkup):
     Description of a reST role.
     """
     def handle_signature(self, sig, signode):
-        # type: (unicode, addnodes.desc_signature) -> unicode
+        # type: (str, addnodes.desc_signature) -> str
         signode += addnodes.desc_name(':%s:' % sig, ':%s:' % sig)
         return sig
 
@@ -129,24 +126,23 @@ class ReSTDomain(Domain):
     }
     initial_data = {
         'objects': {},  # fullname -> docname, objtype
-    }  # type: Dict[unicode, Dict[unicode, Tuple[unicode, ObjType]]]
+    }  # type: Dict[str, Dict[str, Tuple[str, ObjType]]]
 
     def clear_doc(self, docname):
-        # type: (unicode) -> None
+        # type: (str) -> None
         for (typ, name), doc in list(self.data['objects'].items()):
             if doc == docname:
                 del self.data['objects'][typ, name]
 
     def merge_domaindata(self, docnames, otherdata):
-        # type: (List[unicode], Dict) -> None
+        # type: (List[str], Dict) -> None
         # XXX check duplicates
         for (typ, name), doc in otherdata['objects'].items():
             if doc in docnames:
                 self.data['objects'][typ, name] = doc
 
-    def resolve_xref(self, env, fromdocname, builder, typ, target, node,
-                     contnode):
-        # type: (BuildEnvironment, unicode, Builder, unicode, unicode, nodes.Node, nodes.Node) -> nodes.Node  # NOQA
+    def resolve_xref(self, env, fromdocname, builder, typ, target, node, contnode):
+        # type: (BuildEnvironment, str, Builder, str, str, addnodes.pending_xref, nodes.Element) -> nodes.Element  # NOQA
         objects = self.data['objects']
         objtypes = self.objtypes_for_role(typ)
         for objtype in objtypes:
@@ -155,12 +151,12 @@ class ReSTDomain(Domain):
                                     objects[objtype, target],
                                     objtype + '-' + target,
                                     contnode, target + ' ' + objtype)
+        return None
 
-    def resolve_any_xref(self, env, fromdocname, builder, target,
-                         node, contnode):
-        # type: (BuildEnvironment, unicode, Builder, unicode, nodes.Node, nodes.Node) -> List[nodes.Node]  # NOQA
+    def resolve_any_xref(self, env, fromdocname, builder, target, node, contnode):
+        # type: (BuildEnvironment, str, Builder, str, addnodes.pending_xref, nodes.Element) -> List[Tuple[str, nodes.Element]]  # NOQA
         objects = self.data['objects']
-        results = []
+        results = []  # type: List[Tuple[str, nodes.Element]]
         for objtype in self.object_types:
             if (objtype, target) in self.data['objects']:
                 results.append(('rst:' + self.role_for_objtype(objtype),
@@ -171,13 +167,13 @@ class ReSTDomain(Domain):
         return results
 
     def get_objects(self):
-        # type: () -> Iterator[Tuple[unicode, unicode, unicode, unicode, unicode, int]]
-        for (typ, name), docname in iteritems(self.data['objects']):
+        # type: () -> Iterator[Tuple[str, str, str, str, str, int]]
+        for (typ, name), docname in self.data['objects'].items():
             yield name, name, typ, docname, typ + '-' + name, 1
 
 
 def setup(app):
-    # type: (Sphinx) -> Dict[unicode, Any]
+    # type: (Sphinx) -> Dict[str, Any]
     app.add_domain(ReSTDomain)
 
     return {
