@@ -1,50 +1,42 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.environment
     ~~~~~~~~~~~~~~~~~~
 
     Global creation environment.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import os
-import re
-import sys
+import pickle
 import warnings
 from collections import defaultdict
 from copy import copy
+from io import BytesIO
 from os import path
 
-from docutils.utils import get_source_line
-from six import BytesIO, next
-from six.moves import cPickle as pickle
-
 from sphinx import addnodes
-from sphinx.deprecation import RemovedInSphinx20Warning, RemovedInSphinx30Warning
-from sphinx.environment.adapters.indexentries import IndexEntries
+from sphinx.deprecation import RemovedInSphinx30Warning, RemovedInSphinx40Warning
 from sphinx.environment.adapters.toctree import TocTree
 from sphinx.errors import SphinxError, BuildEnvironmentError, DocumentError, ExtensionError
 from sphinx.locale import __
 from sphinx.transforms import SphinxTransformer
-from sphinx.util import get_matching_docs, DownloadFiles, FilenameUniqDict
+from sphinx.util import DownloadFiles, FilenameUniqDict
 from sphinx.util import logging
 from sphinx.util.docutils import LoggingReporter
 from sphinx.util.i18n import find_catalog_files
-from sphinx.util.matching import compile_matchers
 from sphinx.util.nodes import is_translatable
-from sphinx.util.osutil import SEP, relpath
-from sphinx.util.websupport import is_commentable
 
 if False:
     # For type annotation
-    from typing import Any, Callable, Dict, IO, Iterator, List, Optional, Pattern, Set, Tuple, Type, Union, Generator  # NOQA
+    from typing import Any, Callable, Dict, IO, Iterator, List, Optional, Set, Tuple, Union  # NOQA
     from docutils import nodes  # NOQA
     from sphinx.application import Sphinx  # NOQA
     from sphinx.builders import Builder  # NOQA
     from sphinx.config import Config  # NOQA
     from sphinx.domains import Domain  # NOQA
+    from sphinx.project import Project  # NOQA
 
 logger = logging.getLogger(__name__)
 
@@ -61,13 +53,11 @@ default_settings = {
     'halt_level': 5,
     'file_insertion_enabled': True,
     'smartquotes_locales': [],
-}
+}  # type: Dict[str, Any]
 
 # This is increased every time an environment attribute is added
 # or changed to properly invalidate pickle files.
-#
-# NOTE: increase base version by 2 to have distinct numbers for Py2 and 3
-ENV_VERSION = 54 + (sys.version_info[0] - 2)
+ENV_VERSION = 56
 
 # config status
 CONFIG_OK = 1
@@ -85,8 +75,7 @@ CONFIG_CHANGED_REASON = {
 versioning_conditions = {
     'none': False,
     'text': is_translatable,
-    'commentable': is_commentable,
-}  # type: Dict[unicode, Union[bool, Callable]]
+}  # type: Dict[str, Union[bool, Callable]]
 
 
 class NoUri(Exception):
@@ -94,25 +83,26 @@ class NoUri(Exception):
     pass
 
 
-class BuildEnvironment(object):
+class BuildEnvironment:
     """
     The environment in which the ReST files are translated.
     Stores an inventory of cross-file targets and provides doctree
     transformations to resolve links to them.
     """
 
-    domains = None  # type: Dict[unicode, Domain]
+    domains = None  # type: Dict[str, Domain]
 
     # --------- ENVIRONMENT INITIALIZATION -------------------------------------
 
     def __init__(self, app=None):
         # type: (Sphinx) -> None
         self.app = None             # type: Sphinx
-        self.doctreedir = None      # type: unicode
-        self.srcdir = None          # type: unicode
+        self.doctreedir = None      # type: str
+        self.srcdir = None          # type: str
         self.config = None          # type: Config
         self.config_status = None   # type: int
-        self.version = None         # type: Dict[unicode, unicode]
+        self.project = None         # type: Project
+        self.version = None         # type: Dict[str, str]
 
         # the method of doctree versioning; see set_versioning_method
         self.versioning_condition = None  # type: Union[bool, Callable]
@@ -125,68 +115,63 @@ class BuildEnvironment(object):
         self.settings = default_settings.copy()
         self.settings['env'] = self
 
-        # the function to write warning messages with
-        self._warnfunc = None  # type: Callable
-
         # All "docnames" here are /-separated and relative and exclude
         # the source suffix.
 
-        self.found_docs = set()     # type: Set[unicode]
-                                    # contains all existing docnames
-        self.all_docs = {}          # type: Dict[unicode, float]
+        self.all_docs = {}          # type: Dict[str, float]
                                     # docname -> mtime at the time of reading
                                     # contains all read docnames
-        self.dependencies = defaultdict(set)    # type: Dict[unicode, Set[unicode]]
+        self.dependencies = defaultdict(set)    # type: Dict[str, Set[str]]
                                     # docname -> set of dependent file
                                     # names, relative to documentation root
-        self.included = defaultdict(set)        # type: Dict[unicode, Set[unicode]]
+        self.included = defaultdict(set)        # type: Dict[str, Set[str]]
                                     # docname -> set of included file
                                     # docnames included from other documents
-        self.reread_always = set()  # type: Set[unicode]
+        self.reread_always = set()  # type: Set[str]
                                     # docnames to re-read unconditionally on
                                     # next build
 
         # File metadata
-        self.metadata = defaultdict(dict)       # type: Dict[unicode, Dict[unicode, Any]]
+        self.metadata = defaultdict(dict)       # type: Dict[str, Dict[str, Any]]
                                                 # docname -> dict of metadata items
 
         # TOC inventory
-        self.titles = {}            # type: Dict[unicode, nodes.Node]
+        self.titles = {}            # type: Dict[str, nodes.title]
                                     # docname -> title node
-        self.longtitles = {}        # type: Dict[unicode, nodes.Node]
+        self.longtitles = {}        # type: Dict[str, nodes.title]
                                     # docname -> title node; only different if
                                     # set differently with title directive
-        self.tocs = {}              # type: Dict[unicode, nodes.Node]
+        self.tocs = {}              # type: Dict[str, nodes.bullet_list]
                                     # docname -> table of contents nodetree
-        self.toc_num_entries = {}   # type: Dict[unicode, int]
+        self.toc_num_entries = {}   # type: Dict[str, int]
                                     # docname -> number of real entries
 
         # used to determine when to show the TOC
         # in a sidebar (don't show if it's only one item)
-        self.toc_secnumbers = {}    # type: Dict[unicode, Dict[unicode, Tuple[int, ...]]]
+        self.toc_secnumbers = {}    # type: Dict[str, Dict[str, Tuple[int, ...]]]
                                     # docname -> dict of sectionid -> number
-        self.toc_fignumbers = {}    # type: Dict[unicode, Dict[unicode, Dict[unicode, Tuple[int, ...]]]]  # NOQA
+        self.toc_fignumbers = {}    # type: Dict[str, Dict[str, Dict[str, Tuple[int, ...]]]]
                                     # docname -> dict of figtype ->
                                     # dict of figureid -> number
 
-        self.toctree_includes = {}  # type: Dict[unicode, List[unicode]]
+        self.toctree_includes = {}  # type: Dict[str, List[str]]
                                     # docname -> list of toctree includefiles
-        self.files_to_rebuild = {}  # type: Dict[unicode, Set[unicode]]
+        self.files_to_rebuild = {}  # type: Dict[str, Set[str]]
                                     # docname -> set of files
                                     # (containing its TOCs) to rebuild too
-        self.glob_toctrees = set()  # type: Set[unicode]
+        self.glob_toctrees = set()  # type: Set[str]
                                     # docnames that have :glob: toctrees
-        self.numbered_toctrees = set()  # type: Set[unicode]
+        self.numbered_toctrees = set()  # type: Set[str]
                                         # docnames that have :numbered: toctrees
 
         # domain-specific inventories, here to be pickled
-        self.domaindata = {}        # type: Dict[unicode, Dict]
+        self.domaindata = {}        # type: Dict[str, Dict]
                                     # domainname -> domain-specific dict
 
         # Other inventories
-        self.indexentries = {}      # type: Dict[unicode, List[Tuple[unicode, unicode, unicode, unicode, unicode]]]  # NOQA
+        self.indexentries = {}      # type: Dict[str, List[Tuple[str, str, str, str, str]]]
                                     # docname -> list of
-                                    # (type, unicode, target, aliasname)
+                                    # (type, str, target, aliasname)
 
         # these map absolute path -> (docnames, unique filename)
         self.images = FilenameUniqDict()    # type: FilenameUniqDict
@@ -194,14 +179,14 @@ class BuildEnvironment(object):
                                             # filename -> (set of docnames, destination)
 
         # the original URI for images
-        self.original_image_uri = {}  # type: Dict[unicode, unicode]
+        self.original_image_uri = {}  # type: Dict[str, str]
 
         # temporary data storage while reading a document
-        self.temp_data = {}         # type: Dict[unicode, Any]
+        self.temp_data = {}         # type: Dict[str, Any]
         # context for cross-references (e.g. current module or class)
         # this is similar to temp_data, but will for example be copied to
         # attributes of "any" cross references
-        self.ref_context = {}       # type: Dict[unicode, Any]
+        self.ref_context = {}       # type: Dict[str, Any]
 
         # set up environment
         if app:
@@ -226,9 +211,13 @@ class BuildEnvironment(object):
         elif self.srcdir and self.srcdir != app.srcdir:
             raise BuildEnvironmentError(__('source directory has changed'))
 
+        if self.project:
+            app.project.restore(self.project)
+
         self.app = app
         self.doctreedir = app.doctreedir
         self.srcdir = app.srcdir
+        self.project = app.project
         self.version = app.registry.get_envversion(app)
 
         # initialize domains
@@ -273,13 +262,8 @@ class BuildEnvironment(object):
         # Allow to disable by 3rd party extension (workaround)
         self.settings.setdefault('smart_quotes', True)
 
-    def set_warnfunc(self, func):
-        # type: (Callable) -> None
-        warnings.warn('env.set_warnfunc() is now deprecated. Use sphinx.util.logging instead.',
-                      RemovedInSphinx20Warning)
-
     def set_versioning_method(self, method, compare):
-        # type: (unicode, bool) -> None
+        # type: (Union[str, Callable], bool) -> None
         """This sets the doctree versioning method for this environment.
 
         Versioning methods are a builder property; only builders with the same
@@ -287,9 +271,14 @@ class BuildEnvironment(object):
         raise an exception if the user tries to use an environment with an
         incompatible versioning method.
         """
-        if method not in versioning_conditions:
-            raise ValueError('invalid versioning method: %r' % method)
-        condition = versioning_conditions[method]
+        condition = None  # type: Union[bool, Callable]
+        if callable(method):
+            condition = method
+        else:
+            if method not in versioning_conditions:
+                raise ValueError('invalid versioning method: %r' % method)
+            condition = versioning_conditions[method]
+
         if self.versioning_condition not in (None, condition):
             raise SphinxError(__('This environment is incompatible with the '
                                  'selected builder, please choose another '
@@ -297,23 +286,8 @@ class BuildEnvironment(object):
         self.versioning_condition = condition
         self.versioning_compare = compare
 
-    def warn(self, docname, msg, lineno=None, **kwargs):
-        # type: (unicode, unicode, int, Any) -> None
-        """Emit a warning.
-
-        This differs from using ``app.warn()`` in that the warning may not
-        be emitted instantly, but collected for emitting all warnings after
-        the update of the environment.
-        """
-        self.app.warn(msg, location=(docname, lineno), **kwargs)  # type: ignore
-
-    def warn_node(self, msg, node, **kwargs):
-        # type: (unicode, nodes.Node, Any) -> None
-        """Like :meth:`warn`, but with source information taken from *node*."""
-        self._warnfunc(msg, '%s:%s' % get_source_line(node), **kwargs)
-
     def clear_doc(self, docname):
-        # type: (unicode) -> None
+        # type: (str) -> None
         """Remove all traces of a source file in the inventory."""
         if docname in self.all_docs:
             self.all_docs.pop(docname, None)
@@ -324,7 +298,7 @@ class BuildEnvironment(object):
             domain.clear_doc(docname)
 
     def merge_info_from(self, docnames, other, app):
-        # type: (List[unicode], BuildEnvironment, Sphinx) -> None
+        # type: (List[str], BuildEnvironment, Sphinx) -> None
         """Merge global information gathered about *docnames* while reading them
         from the *other* environment.
 
@@ -337,29 +311,20 @@ class BuildEnvironment(object):
             if docname in other.reread_always:
                 self.reread_always.add(docname)
 
-        for version, changes in other.versionchanges.items():
-            self.versionchanges.setdefault(version, []).extend(
-                change for change in changes if change[1] in docnames)
-
         for domainname, domain in self.domains.items():
             domain.merge_domaindata(docnames, other.domaindata[domainname])
         app.emit('env-merge-info', self, docnames, other)
 
     def path2doc(self, filename):
-        # type: (unicode) -> Optional[unicode]
+        # type: (str) -> Optional[str]
         """Return the docname for the filename if the file is document.
 
         *filename* should be absolute or relative to the source directory.
         """
-        if filename.startswith(self.srcdir):
-            filename = relpath(filename, self.srcdir)
-        for suffix in self.config.source_suffix:
-            if filename.endswith(suffix):
-                return filename[:-len(suffix)]
-        return None
+        return self.project.path2doc(filename)
 
     def doc2path(self, docname, base=True, suffix=None):
-        # type: (unicode, Union[bool, unicode], unicode) -> unicode
+        # type: (str, Union[bool, str], str) -> str
         """Return the filename for the document name.
 
         If *base* is True, return absolute path under self.srcdir.
@@ -367,24 +332,23 @@ class BuildEnvironment(object):
         If *base* is a path string, return absolute path under that.
         If *suffix* is not None, add it instead of config.source_suffix.
         """
-        docname = docname.replace(SEP, path.sep)
-        if suffix is None:
-            # Use first candidate if there is not a file for any suffix
-            suffix = next(iter(self.config.source_suffix))
-            for candidate_suffix in self.config.source_suffix:
-                if path.isfile(path.join(self.srcdir, docname) +
-                               candidate_suffix):
-                    suffix = candidate_suffix
-                    break
-        if base is True:
-            return path.join(self.srcdir, docname) + suffix
-        elif base is None:
-            return docname + suffix
-        else:
-            return path.join(base, docname) + suffix  # type: ignore
+        if suffix:
+            warnings.warn('The suffix argument for doc2path() is deprecated.',
+                          RemovedInSphinx40Warning)
+        if base not in (True, None):
+            warnings.warn('The string style base argument for doc2path() is deprecated.',
+                          RemovedInSphinx40Warning)
+
+        pathname = self.project.doc2path(docname, base is True)
+        if suffix:
+            filename, _ = path.splitext(pathname)
+            pathname = filename + suffix
+        if base and base is not True:
+            pathname = path.join(base, pathname)  # type: ignore
+        return pathname
 
     def relfn2path(self, filename, docname=None):
-        # type: (unicode, unicode) -> Tuple[unicode, unicode]
+        # type: (str, str) -> Tuple[str, str]
         """Return paths to a file referenced from a document, relative to
         documentation root and absolute.
 
@@ -398,15 +362,15 @@ class BuildEnvironment(object):
             docdir = path.dirname(self.doc2path(docname or self.docname,
                                                 base=None))
             rel_fn = path.join(docdir, filename)
-        try:
-            # the path.abspath() might seem redundant, but otherwise artifacts
-            # such as ".." will remain in the path
-            return rel_fn, path.abspath(path.join(self.srcdir, rel_fn))
-        except UnicodeDecodeError:
-            # the source directory is a bytestring with non-ASCII characters;
-            # let's try to encode the rel_fn in the file system encoding
-            enc_rel_fn = rel_fn.encode(sys.getfilesystemencoding())
-            return rel_fn, path.abspath(path.join(self.srcdir, enc_rel_fn))
+        # the path.abspath() might seem redundant, but otherwise artifacts
+        # such as ".." will remain in the path
+        return rel_fn, path.abspath(path.join(self.srcdir, rel_fn))
+
+    @property
+    def found_docs(self):
+        # type: () -> Set[str]
+        """contains all existing docnames."""
+        return self.project.docnames
 
     def find_files(self, config, builder):
         # type: (Config, Builder) -> None
@@ -414,19 +378,10 @@ class BuildEnvironment(object):
         self.found_docs.
         """
         try:
-            matchers = compile_matchers(
-                config.exclude_patterns[:] +
-                config.templates_path +
-                builder.get_asset_paths() +
-                ['**/_sources', '.#*', '**/.#*', '*.lproj/**']
-            )
-            self.found_docs = set()
-            for docname in get_matching_docs(self.srcdir, config.source_suffix,  # type: ignore
-                                             exclude_matchers=matchers):
-                if os.access(self.doc2path(docname), os.R_OK):
-                    self.found_docs.add(docname)
-                else:
-                    logger.warning(__("document not readable. Ignored."), location=docname)
+            exclude_paths = (self.config.exclude_patterns +
+                             self.config.templates_path +
+                             builder.get_asset_paths())
+            self.project.discover(exclude_paths)
 
             # Current implementation is applying translated messages in the reading
             # phase.Therefore, in order to apply the updated message catalog, it is
@@ -445,17 +400,17 @@ class BuildEnvironment(object):
                         self.config.gettext_compact)
                     for filename in catalog_files:
                         self.dependencies[docname].add(filename)
-        except EnvironmentError as exc:
+        except OSError as exc:
             raise DocumentError(__('Failed to scan documents in %s: %r') % (self.srcdir, exc))
 
     def get_outdated_files(self, config_changed):
-        # type: (bool) -> Tuple[Set[unicode], Set[unicode], Set[unicode]]
+        # type: (bool) -> Tuple[Set[str], Set[str], Set[str]]
         """Return (added, changed, removed) sets."""
         # clear all files no longer present
         removed = set(self.all_docs) - self.found_docs
 
-        added = set()  # type: Set[unicode]
-        changed = set()  # type: Set[unicode]
+        added = set()  # type: Set[str]
+        changed = set()  # type: Set[str]
 
         if config_changed:
             # config values affect e.g. substitutions
@@ -466,8 +421,8 @@ class BuildEnvironment(object):
                     added.add(docname)
                     continue
                 # if the doctree file is not there, rebuild
-                if not path.isfile(self.doc2path(docname, self.doctreedir,
-                                                 '.doctree')):
+                filename = path.join(self.doctreedir, docname + '.doctree')
+                if not path.isfile(filename):
                     changed.add(docname)
                     continue
                 # check the "reread always" list
@@ -492,7 +447,7 @@ class BuildEnvironment(object):
                         if depmtime > mtime:
                             changed.add(docname)
                             break
-                    except EnvironmentError:
+                    except OSError:
                         # give it another chance
                         changed.add(docname)
                         break
@@ -500,8 +455,8 @@ class BuildEnvironment(object):
         return added, changed, removed
 
     def check_dependents(self, app, already):
-        # type: (Sphinx, Set[unicode]) -> Iterator[unicode]
-        to_rewrite = []  # type: List[unicode]
+        # type: (Sphinx, Set[str]) -> Iterator[str]
+        to_rewrite = []  # type: List[str]
         for docnames in app.emit('env-get-updated', self):
             to_rewrite.extend(docnames)
         for docname in set(to_rewrite):
@@ -511,7 +466,7 @@ class BuildEnvironment(object):
     # --------- SINGLE FILE READING --------------------------------------------
 
     def prepare_settings(self, docname):
-        # type: (unicode) -> None
+        # type: (str) -> None
         """Prepare to set up environment for reading."""
         self.temp_data['docname'] = docname
         # defaults to the global default, but can be re-set in a document
@@ -523,12 +478,12 @@ class BuildEnvironment(object):
 
     @property
     def docname(self):
-        # type: () -> unicode
+        # type: () -> str
         """Returns the docname of the document currently being parsed."""
         return self.temp_data['docname']
 
     def new_serialno(self, category=''):
-        # type: (unicode) -> int
+        # type: (str) -> int
         """Return a serial number, e.g. for index entry targets.
 
         The number is guaranteed to be unique in the current document.
@@ -539,7 +494,7 @@ class BuildEnvironment(object):
         return cur
 
     def note_dependency(self, filename):
-        # type: (unicode) -> None
+        # type: (str) -> None
         """Add *filename* as a dependency of the current document.
 
         This means that the document will be rebuilt if this file changes.
@@ -549,7 +504,7 @@ class BuildEnvironment(object):
         self.dependencies[self.docname].add(filename)
 
     def note_included(self, filename):
-        # type: (unicode) -> None
+        # type: (str) -> None
         """Add *filename* as a included from other document.
 
         This means the document is not orphaned.
@@ -565,34 +520,8 @@ class BuildEnvironment(object):
         """
         self.reread_always.add(self.docname)
 
-    def note_toctree(self, docname, toctreenode):
-        # type: (unicode, addnodes.toctree) -> None
-        """Note a TOC tree directive in a document and gather information about
-        file relations from it.
-        """
-        warnings.warn('env.note_toctree() is deprecated. '
-                      'Use sphinx.environment.adapters.toctree.TocTree instead.',
-                      RemovedInSphinx20Warning)
-        TocTree(self).note(docname, toctreenode)
-
-    def get_toc_for(self, docname, builder):
-        # type: (unicode, Builder) -> Dict[unicode, nodes.Node]
-        """Return a TOC nodetree -- for use on the same page only!"""
-        warnings.warn('env.get_toc_for() is deprecated. '
-                      'Use sphinx.environment.adapters.toctre.TocTree instead.',
-                      RemovedInSphinx20Warning)
-        return TocTree(self).get_toc_for(docname, builder)
-
-    def get_toctree_for(self, docname, builder, collapse, **kwds):
-        # type: (unicode, Builder, bool, Any) -> addnodes.toctree
-        """Return the global TOC nodetree."""
-        warnings.warn('env.get_toctree_for() is deprecated. '
-                      'Use sphinx.environment.adapters.toctre.TocTree instead.',
-                      RemovedInSphinx20Warning)
-        return TocTree(self).get_toctree_for(docname, builder, collapse, **kwds)
-
     def get_domain(self, domainname):
-        # type: (unicode) -> Domain
+        # type: (str) -> Domain
         """Return the domain instance with the specified name.
 
         Raises an ExtensionError if the domain is not registered.
@@ -605,10 +534,10 @@ class BuildEnvironment(object):
     # --------- RESOLVING REFERENCES AND TOCTREES ------------------------------
 
     def get_doctree(self, docname):
-        # type: (unicode) -> nodes.Node
+        # type: (str) -> nodes.document
         """Read the doctree for a file from the pickle and return it."""
-        doctree_filename = self.doc2path(docname, self.doctreedir, '.doctree')
-        with open(doctree_filename, 'rb') as f:
+        filename = path.join(self.doctreedir, docname + '.doctree')
+        with open(filename, 'rb') as f:
             doctree = pickle.load(f)
         doctree.settings.env = self
         doctree.reporter = LoggingReporter(self.doc2path(docname))
@@ -616,7 +545,7 @@ class BuildEnvironment(object):
 
     def get_and_resolve_doctree(self, docname, builder, doctree=None,
                                 prune_toctrees=True, includehidden=False):
-        # type: (unicode, Builder, nodes.Node, bool, bool) -> nodes.Node
+        # type: (str, Builder, nodes.document, bool, bool) -> nodes.document
         """Read the doctree from the pickle, resolve cross-references and
         toctrees and return it.
         """
@@ -640,7 +569,7 @@ class BuildEnvironment(object):
 
     def resolve_toctree(self, docname, builder, toctree, prune=True, maxdepth=0,
                         titles_only=False, collapse=False, includehidden=False):
-        # type: (unicode, Builder, addnodes.toctree, bool, int, bool, bool, bool) -> nodes.Node
+        # type: (str, Builder, addnodes.toctree, bool, int, bool, bool, bool) -> nodes.Node
         """Resolve a *toctree* node into individual bullet lists with titles
         as items, returning None (if no containing titles are found) or
         a new node.
@@ -657,11 +586,11 @@ class BuildEnvironment(object):
                                      includehidden)
 
     def resolve_references(self, doctree, fromdocname, builder):
-        # type: (nodes.Node, unicode, Builder) -> None
+        # type: (nodes.document, str, Builder) -> None
         self.apply_post_transforms(doctree, fromdocname)
 
     def apply_post_transforms(self, doctree, docname):
-        # type: (nodes.Node, unicode) -> None
+        # type: (nodes.document, str) -> None
         """Apply all post-transforms."""
         try:
             # set env.docname during applying post-transforms
@@ -678,22 +607,12 @@ class BuildEnvironment(object):
         # allow custom references to be resolved
         self.app.emit('doctree-resolved', doctree, docname)
 
-    def create_index(self, builder, group_entries=True,
-                     _fixre=re.compile(r'(.*) ([(][^()]*[)])')):
-        # type: (Builder, bool, Pattern) -> List[Tuple[unicode, List[Tuple[unicode, List[unicode]]]]]  # NOQA
-        warnings.warn('env.create_index() is deprecated. '
-                      'Use sphinx.environment.adapters.indexentreis.IndexEntries instead.',
-                      RemovedInSphinx20Warning)
-        return IndexEntries(self).create_index(builder,
-                                               group_entries=group_entries,
-                                               _fixre=_fixre)
-
     def collect_relations(self):
-        # type: () -> Dict[unicode, List[unicode]]
+        # type: () -> Dict[str, List[str]]
         traversed = set()
 
         def traverse_toctree(parent, docname):
-            # type: (unicode, unicode) -> Iterator[Tuple[unicode, unicode]]
+            # type: (str, str) -> Iterator[Tuple[str, str]]
             if parent == docname:
                 logger.warning(__('self referenced toctree found. Ignored.'), location=docname)
                 return
@@ -747,42 +666,42 @@ class BuildEnvironment(object):
     # --------- METHODS FOR COMPATIBILITY --------------------------------------
 
     def update(self, config, srcdir, doctreedir):
-        # type: (Config, unicode, unicode) -> List[unicode]
+        # type: (Config, str, str) -> List[str]
         warnings.warn('env.update() is deprecated. Please use builder.read() instead.',
-                      RemovedInSphinx30Warning)
+                      RemovedInSphinx30Warning, stacklevel=2)
         return self.app.builder.read()
 
     def _read_serial(self, docnames, app):
-        # type: (List[unicode], Sphinx) -> None
+        # type: (List[str], Sphinx) -> None
         warnings.warn('env._read_serial() is deprecated. Please use builder.read() instead.',
-                      RemovedInSphinx30Warning)
+                      RemovedInSphinx30Warning, stacklevel=2)
         return self.app.builder._read_serial(docnames)
 
     def _read_parallel(self, docnames, app, nproc):
-        # type: (List[unicode], Sphinx, int) -> None
+        # type: (List[str], Sphinx, int) -> None
         warnings.warn('env._read_parallel() is deprecated. Please use builder.read() instead.',
-                      RemovedInSphinx30Warning)
+                      RemovedInSphinx30Warning, stacklevel=2)
         return self.app.builder._read_parallel(docnames, nproc)
 
     def read_doc(self, docname, app=None):
-        # type: (unicode, Sphinx) -> None
+        # type: (str, Sphinx) -> None
         warnings.warn('env.read_doc() is deprecated. Please use builder.read_doc() instead.',
-                      RemovedInSphinx30Warning)
+                      RemovedInSphinx30Warning, stacklevel=2)
         self.app.builder.read_doc(docname)
 
     def write_doctree(self, docname, doctree):
-        # type: (unicode, nodes.Node) -> None
+        # type: (str, nodes.document) -> None
         warnings.warn('env.write_doctree() is deprecated. '
                       'Please use builder.write_doctree() instead.',
-                      RemovedInSphinx30Warning)
+                      RemovedInSphinx30Warning, stacklevel=2)
         self.app.builder.write_doctree(docname, doctree)
 
     @property
     def _nitpick_ignore(self):
-        # type: () -> List[unicode]
+        # type: () -> List[str]
         warnings.warn('env._nitpick_ignore is deprecated. '
                       'Please use config.nitpick_ignore instead.',
-                      RemovedInSphinx30Warning)
+                      RemovedInSphinx30Warning, stacklevel=2)
         return self.config.nitpick_ignore
 
     @staticmethod
@@ -790,13 +709,13 @@ class BuildEnvironment(object):
         # type: (IO, Sphinx) -> BuildEnvironment
         warnings.warn('BuildEnvironment.load() is deprecated. '
                       'Please use pickle.load() instead.',
-                      RemovedInSphinx30Warning)
+                      RemovedInSphinx30Warning, stacklevel=2)
         try:
             env = pickle.load(f)
         except Exception as exc:
             # This can happen for example when the pickle is from a
             # different version of Sphinx.
-            raise IOError(exc)
+            raise OSError(exc)
         if app:
             env.app = app
             env.config.values = app.config.values
@@ -804,19 +723,19 @@ class BuildEnvironment(object):
 
     @classmethod
     def loads(cls, string, app=None):
-        # type: (unicode, Sphinx) -> BuildEnvironment
+        # type: (bytes, Sphinx) -> BuildEnvironment
         warnings.warn('BuildEnvironment.loads() is deprecated. '
                       'Please use pickle.loads() instead.',
-                      RemovedInSphinx30Warning)
+                      RemovedInSphinx30Warning, stacklevel=2)
         io = BytesIO(string)
         return cls.load(io, app)
 
     @classmethod
     def frompickle(cls, filename, app):
-        # type: (unicode, Sphinx) -> BuildEnvironment
+        # type: (str, Sphinx) -> BuildEnvironment
         warnings.warn('BuildEnvironment.frompickle() is deprecated. '
                       'Please use pickle.load() instead.',
-                      RemovedInSphinx30Warning)
+                      RemovedInSphinx30Warning, stacklevel=2)
         with open(filename, 'rb') as f:
             return cls.load(f, app)
 
@@ -825,40 +744,40 @@ class BuildEnvironment(object):
         # type: (BuildEnvironment, IO) -> None
         warnings.warn('BuildEnvironment.dump() is deprecated. '
                       'Please use pickle.dump() instead.',
-                      RemovedInSphinx30Warning)
+                      RemovedInSphinx30Warning, stacklevel=2)
         pickle.dump(env, f, pickle.HIGHEST_PROTOCOL)
 
     @classmethod
     def dumps(cls, env):
-        # type: (BuildEnvironment) -> unicode
+        # type: (BuildEnvironment) -> bytes
         warnings.warn('BuildEnvironment.dumps() is deprecated. '
                       'Please use pickle.dumps() instead.',
-                      RemovedInSphinx30Warning)
+                      RemovedInSphinx30Warning, stacklevel=2)
         io = BytesIO()
         cls.dump(env, io)
         return io.getvalue()
 
     def topickle(self, filename):
-        # type: (unicode) -> None
+        # type: (str) -> None
         warnings.warn('env.topickle() is deprecated. '
                       'Please use pickle.dump() instead.',
-                      RemovedInSphinx30Warning)
+                      RemovedInSphinx30Warning, stacklevel=2)
         with open(filename, 'wb') as f:
             self.dump(self, f)
 
     @property
     def versionchanges(self):
-        # type: () -> Dict[unicode, List[Tuple[unicode, unicode, int, unicode, unicode, unicode]]]  # NOQA
+        # type: () -> Dict[str, List[Tuple[str, str, int, str, str, str]]]
         warnings.warn('env.versionchanges() is deprecated. '
                       'Please use ChangeSetDomain instead.',
-                      RemovedInSphinx30Warning)
+                      RemovedInSphinx30Warning, stacklevel=2)
         return self.domaindata['changeset']['changes']
 
     def note_versionchange(self, type, version, node, lineno):
-        # type: (unicode, unicode, nodes.Node, int) -> None
+        # type: (str, str, addnodes.versionmodified, int) -> None
         warnings.warn('env.note_versionchange() is deprecated. '
                       'Please use ChangeSetDomain.note_changeset() instead.',
-                      RemovedInSphinx30Warning)
+                      RemovedInSphinx30Warning, stacklevel=2)
         node['type'] = type
         node['version'] = version
         node.line = lineno
