@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.ext.autodoc.directive
     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -8,19 +7,18 @@
 """
 
 from docutils import nodes
-from docutils.parsers.rst import Directive
-from docutils.statemachine import ViewList
+from docutils.statemachine import StringList
 from docutils.utils import assemble_option_dict
 
 from sphinx.ext.autodoc import Options, get_documenters
 from sphinx.util import logging
-from sphinx.util.docutils import switch_source_input
+from sphinx.util.docutils import SphinxDirective, switch_source_input
 from sphinx.util.nodes import nested_parse_with_titles
 
 if False:
     # For type annotation
-    from typing import Any, Dict, List, Set, Type  # NOQA
-    from docutils.statemachine import State, StateMachine, StringList  # NOQA
+    from typing import Callable, Dict, List, Set, Type  # NOQA
+    from docutils.parsers.rst.state import RSTState  # NOQA
     from docutils.utils import Reporter  # NOQA
     from sphinx.config import Config  # NOQA
     from sphinx.environment import BuildEnvironment  # NOQA
@@ -32,18 +30,23 @@ logger = logging.getLogger(__name__)
 # common option names for autodoc directives
 AUTODOC_DEFAULT_OPTIONS = ['members', 'undoc-members', 'inherited-members',
                            'show-inheritance', 'private-members', 'special-members',
-                           'ignore-module-all']
+                           'ignore-module-all', 'exclude-members', 'member-order']
 
 
-class DummyOptionSpec(object):
+class DummyOptionSpec(dict):
     """An option_spec allows any options."""
 
+    def __bool__(self):
+        # type: () -> bool
+        """Behaves like some options are defined."""
+        return True
+
     def __getitem__(self, key):
-        # type: (Any) -> Any
+        # type: (str) -> Callable[[str], str]
         return lambda x: x
 
 
-class DocumenterBridge(object):
+class DocumenterBridge:
     """A parameters container for Documenters."""
 
     def __init__(self, env, reporter, options, lineno):
@@ -52,11 +55,11 @@ class DocumenterBridge(object):
         self.reporter = reporter
         self.genopt = options
         self.lineno = lineno
-        self.filename_set = set()  # type: Set[unicode]
-        self.result = ViewList()
+        self.filename_set = set()  # type: Set[str]
+        self.result = StringList()
 
     def warn(self, msg):
-        # type: (unicode) -> None
+        # type: (str) -> None
         logger.warning(msg, location=(self.env.docname, self.lineno))
 
 
@@ -68,18 +71,18 @@ def process_documenter_options(documenter, config, options):
             continue
         else:
             negated = options.pop('no-' + name, True) is None
-            if name in config.autodoc_default_flags and not negated:
-                options[name] = None
+            if name in config.autodoc_default_options and not negated:
+                options[name] = config.autodoc_default_options[name]
 
     return Options(assemble_option_dict(options.items(), documenter.option_spec))
 
 
 def parse_generated_content(state, content, documenter):
-    # type: (State, StringList, Documenter) -> List[nodes.Node]
+    # type: (RSTState, StringList, Documenter) -> List[nodes.Node]
     """Parse a generated content by Documenter."""
     with switch_source_input(state, content):
         if documenter.titles_allowed:
-            node = nodes.section()
+            node = nodes.section()  # type: nodes.Element
             # necessary so that the child nodes get the right source/line set
             node.document = state.document
             nested_parse_with_titles(state, content, node)
@@ -91,7 +94,7 @@ def parse_generated_content(state, content, documenter):
         return node.children
 
 
-class AutodocDirective(Directive):
+class AutodocDirective(SphinxDirective):
     """A directive class for all autodoc directives. It works as a dispatcher of Documenters.
 
     It invokes a Documenter on running. After the processing, it parses and returns
@@ -105,22 +108,21 @@ class AutodocDirective(Directive):
 
     def run(self):
         # type: () -> List[nodes.Node]
-        env = self.state.document.settings.env
         reporter = self.state.document.reporter
 
         try:
-            source, lineno = reporter.get_source_and_line(self.lineno)
+            source, lineno = reporter.get_source_and_line(self.lineno)  # type: ignore
         except AttributeError:
             source, lineno = (None, None)
         logger.debug('[autodoc] %s:%s: input:\n%s', source, lineno, self.block_text)
 
         # look up target Documenter
         objtype = self.name[4:]  # strip prefix (auto-).
-        doccls = get_documenters(env.app)[objtype]
+        doccls = get_documenters(self.env.app)[objtype]
 
         # process the options with the selected documenter's option_spec
         try:
-            documenter_options = process_documenter_options(doccls, env.config, self.options)
+            documenter_options = process_documenter_options(doccls, self.config, self.options)
         except (KeyError, ValueError, TypeError) as exc:
             # an option is either unknown or has a wrong type
             logger.error('An option to %s is either unknown or has an invalid value: %s' %
@@ -128,7 +130,7 @@ class AutodocDirective(Directive):
             return []
 
         # generate the output
-        params = DocumenterBridge(env, reporter, documenter_options, lineno)
+        params = DocumenterBridge(self.env, reporter, documenter_options, lineno)
         documenter = doccls(params, self.arguments[0])
         documenter.generate(more_content=self.content)
         if not params.result:
