@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.ext.napoleon.docstring
     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -7,19 +6,22 @@
     Classes for docstring parsing and formatting.
 
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-import collections
 import inspect
 import re
-
-from six import string_types, u
-from six.moves import range
+from functools import partial
 
 from sphinx.ext.napoleon.iterators import modify_iter
-from sphinx.util.pycompat import UnicodeMixin
+from sphinx.locale import _
+
+if False:
+    # For type annotation
+    from typing import Any, Callable, Dict, List, Tuple, Type, Union  # NOQA
+    from sphinx.application import Sphinx  # NOQA
+    from sphinx.config import Config as SphinxConfig  # NOQA
 
 
 _directive_regex = re.compile(r'\.\. \S+::')
@@ -27,7 +29,7 @@ _google_section_regex = re.compile(r'^(\s|\w)+:\s*$')
 _google_typed_arg_regex = re.compile(r'\s*(.+?)\s*\(\s*(.*[^\s]+)\s*\)')
 _numpy_section_regex = re.compile(r'^[=\-`:\'"~^_*+#<>]{2,}\s*$')
 _single_colon_regex = re.compile(r'(?<!:):(?!:)')
-_xref_regex = re.compile(r'(:\w+:\S+:`.+?`|:\S+:`.+?`|`.+?`)')
+_xref_regex = re.compile(r'(:(?:[a-zA-Z0-9]+[\-_+:.])*[a-zA-Z0-9]+:`.+?`)')
 _bullet_list_regex = re.compile(r'^(\*|\+|\-)(\s+\S|\s*$)')
 _enumerated_list_regex = re.compile(
     r'^(?P<paren>\()?'
@@ -35,7 +37,7 @@ _enumerated_list_regex = re.compile(
     r'(?(paren)\)|\.)(\s+\S|\s*$)')
 
 
-class GoogleDocstring(UnicodeMixin):
+class GoogleDocstring:
     """Convert Google style docstrings to reStructuredText.
 
     Parameters
@@ -97,21 +99,26 @@ class GoogleDocstring(UnicodeMixin):
     <BLANKLINE>
 
     """
+
+    _name_rgx = re.compile(r"^\s*((?::(?P<role>\S+):)?`(?P<name>[a-zA-Z0-9_.-]+)`|"
+                           r" (?P<name2>[a-zA-Z0-9_.-]+))\s*", re.X)
+
     def __init__(self, docstring, config=None, app=None, what='', name='',
                  obj=None, options=None):
+        # type: (Union[str, List[str]], SphinxConfig, Sphinx, str, str, Any, Any) -> None
         self._config = config
         self._app = app
 
         if not self._config:
             from sphinx.ext.napoleon import Config
-            self._config = self._app and self._app.config or Config()
+            self._config = self._app and self._app.config or Config()  # type: ignore
 
         if not what:
             if inspect.isclass(obj):
                 what = 'class'
             elif inspect.ismodule(obj):
                 what = 'module'
-            elif isinstance(obj, collections.Callable):
+            elif callable(obj):
                 what = 'function'
             else:
                 what = 'object'
@@ -120,26 +127,33 @@ class GoogleDocstring(UnicodeMixin):
         self._name = name
         self._obj = obj
         self._opt = options
-        if isinstance(docstring, string_types):
-            docstring = docstring.splitlines()
-        self._lines = docstring
-        self._line_iter = modify_iter(docstring, modifier=lambda s: s.rstrip())
-        self._parsed_lines = []
+        if isinstance(docstring, str):
+            lines = docstring.splitlines()
+        else:
+            lines = docstring
+        self._line_iter = modify_iter(lines, modifier=lambda s: s.rstrip())
+        self._parsed_lines = []  # type: List[str]
         self._is_in_section = False
         self._section_indent = 0
         if not hasattr(self, '_directive_sections'):
-            self._directive_sections = []
+            self._directive_sections = []  # type: List[str]
         if not hasattr(self, '_sections'):
             self._sections = {
                 'args': self._parse_parameters_section,
                 'arguments': self._parse_parameters_section,
+                'attention': partial(self._parse_admonition, 'attention'),
                 'attributes': self._parse_attributes_section,
+                'caution': partial(self._parse_admonition, 'caution'),
+                'danger': partial(self._parse_admonition, 'danger'),
+                'error': partial(self._parse_admonition, 'error'),
                 'example': self._parse_examples_section,
                 'examples': self._parse_examples_section,
+                'hint': partial(self._parse_admonition, 'hint'),
+                'important': partial(self._parse_admonition, 'important'),
                 'keyword args': self._parse_keyword_arguments_section,
                 'keyword arguments': self._parse_keyword_arguments_section,
                 'methods': self._parse_methods_section,
-                'note': self._parse_note_section,
+                'note': partial(self._parse_admonition, 'note'),
                 'notes': self._parse_notes_section,
                 'other parameters': self._parse_other_parameters_section,
                 'parameters': self._parse_parameters_section,
@@ -148,16 +162,21 @@ class GoogleDocstring(UnicodeMixin):
                 'raises': self._parse_raises_section,
                 'references': self._parse_references_section,
                 'see also': self._parse_see_also_section,
-                'todo': self._parse_todo_section,
-                'warning': self._parse_warning_section,
-                'warnings': self._parse_warning_section,
+                'tip': partial(self._parse_admonition, 'tip'),
+                'todo': partial(self._parse_admonition, 'todo'),
+                'warning': partial(self._parse_admonition, 'warning'),
+                'warnings': partial(self._parse_admonition, 'warning'),
                 'warns': self._parse_warns_section,
                 'yield': self._parse_yields_section,
                 'yields': self._parse_yields_section,
-            }
+            }  # type: Dict[str, Callable]
+
+        self._load_custom_sections()
+
         self._parse()
 
-    def __unicode__(self):
+    def __str__(self):
+        # type: () -> str
         """Return the parsed docstring in reStructuredText format.
 
         Returns
@@ -166,9 +185,10 @@ class GoogleDocstring(UnicodeMixin):
             Unicode version of the docstring.
 
         """
-        return u('\n').join(self.lines())
+        return '\n'.join(self.lines())
 
     def lines(self):
+        # type: () -> List[str]
         """Return the parsed lines of the docstring in reStructuredText format.
 
         Returns
@@ -180,6 +200,7 @@ class GoogleDocstring(UnicodeMixin):
         return self._parsed_lines
 
     def _consume_indented_block(self, indent=1):
+        # type: (int) -> List[str]
         lines = []
         line = self._line_iter.peek()
         while(not self._is_section_break() and
@@ -189,6 +210,7 @@ class GoogleDocstring(UnicodeMixin):
         return lines
 
     def _consume_contiguous(self):
+        # type: () -> List[str]
         lines = []
         while (self._line_iter.has_next() and
                self._line_iter.peek() and
@@ -197,6 +219,7 @@ class GoogleDocstring(UnicodeMixin):
         return lines
 
     def _consume_empty(self):
+        # type: () -> List[str]
         lines = []
         line = self._line_iter.peek()
         while self._line_iter.has_next() and not line:
@@ -205,6 +228,7 @@ class GoogleDocstring(UnicodeMixin):
         return lines
 
     def _consume_field(self, parse_type=True, prefer_type=False):
+        # type: (bool, bool) -> Tuple[str, str, List[str]]
         line = next(self._line_iter)
 
         before, colon, after = self._partition_field_on_colon(line)
@@ -221,11 +245,12 @@ class GoogleDocstring(UnicodeMixin):
         if prefer_type and not _type:
             _type, _name = _name, _type
         indent = self._get_indent(line) + 1
-        _desc = [_desc] + self._dedent(self._consume_indented_block(indent))
-        _desc = self.__class__(_desc, self._config).lines()
-        return _name, _type, _desc
+        _descs = [_desc] + self._dedent(self._consume_indented_block(indent))
+        _descs = self.__class__(_descs, self._config).lines()
+        return _name, _type, _descs
 
     def _consume_fields(self, parse_type=True, prefer_type=False):
+        # type: (bool, bool) -> List[Tuple[str, str, List[str]]]
         self._consume_empty()
         fields = []
         while not self._is_section_break():
@@ -235,15 +260,18 @@ class GoogleDocstring(UnicodeMixin):
         return fields
 
     def _consume_inline_attribute(self):
+        # type: () -> Tuple[str, List[str]]
         line = next(self._line_iter)
         _type, colon, _desc = self._partition_field_on_colon(line)
-        if not colon:
+        if not colon or not _desc:
             _type, _desc = _desc, _type
-        _desc = [_desc] + self._dedent(self._consume_to_end())
-        _desc = self.__class__(_desc, self._config).lines()
-        return _type, _desc
+            _desc += colon
+        _descs = [_desc] + self._dedent(self._consume_to_end())
+        _descs = self.__class__(_descs, self._config).lines()
+        return _type, _descs
 
     def _consume_returns_section(self):
+        # type: () -> List[Tuple[str, str, List[str]]]
         lines = self._dedent(self._consume_to_next_section())
         if lines:
             before, colon, after = self._partition_field_on_colon(lines[0])
@@ -263,10 +291,12 @@ class GoogleDocstring(UnicodeMixin):
             return []
 
     def _consume_usage_section(self):
+        # type: () -> List[str]
         lines = self._dedent(self._consume_to_next_section())
         return lines
 
     def _consume_section_header(self):
+        # type: () -> str
         section = next(self._line_iter)
         stripped_section = section.strip(':')
         if stripped_section.lower() in self._sections:
@@ -274,12 +304,14 @@ class GoogleDocstring(UnicodeMixin):
         return section
 
     def _consume_to_end(self):
+        # type: () -> List[str]
         lines = []
         while self._line_iter.has_next():
             lines.append(next(self._line_iter))
         return lines
 
     def _consume_to_next_section(self):
+        # type: () -> List[str]
         self._consume_empty()
         lines = []
         while not self._is_section_break():
@@ -287,6 +319,7 @@ class GoogleDocstring(UnicodeMixin):
         return lines + self._consume_empty()
 
     def _dedent(self, lines, full=False):
+        # type: (List[str], bool) -> List[str]
         if full:
             return [line.lstrip() for line in lines]
         else:
@@ -294,6 +327,7 @@ class GoogleDocstring(UnicodeMixin):
             return [line[min_indent:] for line in lines]
 
     def _escape_args_and_kwargs(self, name):
+        # type: (str) -> str
         if name[:2] == '**':
             return r'\*\*' + name[2:]
         elif name[:1] == '*':
@@ -302,6 +336,7 @@ class GoogleDocstring(UnicodeMixin):
             return name
 
     def _fix_field_desc(self, desc):
+        # type: (List[str]) -> List[str]
         if self._is_list(desc):
             desc = [''] + desc
         elif desc[0].endswith('::'):
@@ -315,6 +350,7 @@ class GoogleDocstring(UnicodeMixin):
         return desc
 
     def _format_admonition(self, admonition, lines):
+        # type: (str, List[str]) -> List[str]
         lines = self._strip_empty(lines)
         if len(lines) == 1:
             return ['.. %s:: %s' % (admonition, lines[0].strip()), '']
@@ -325,6 +361,7 @@ class GoogleDocstring(UnicodeMixin):
             return ['.. %s::' % admonition, '']
 
     def _format_block(self, prefix, lines, padding=None):
+        # type: (str, List[str], str) -> List[str]
         if lines:
             if padding is None:
                 padding = ' ' * len(prefix)
@@ -342,6 +379,7 @@ class GoogleDocstring(UnicodeMixin):
 
     def _format_docutils_params(self, fields, field_role='param',
                                 type_role='type'):
+        # type: (List[Tuple[str, str, List[str]]], str, str) -> List[str]
         lines = []
         for _name, _type, _desc in fields:
             _desc = self._strip_empty(_desc)
@@ -357,6 +395,7 @@ class GoogleDocstring(UnicodeMixin):
         return lines + ['']
 
     def _format_field(self, _name, _type, _desc):
+        # type: (str, str, List[str]) -> List[str]
         _desc = self._strip_empty(_desc)
         has_desc = any(_desc)
         separator = has_desc and ' -- ' or ''
@@ -386,10 +425,11 @@ class GoogleDocstring(UnicodeMixin):
             return [field]
 
     def _format_fields(self, field_type, fields):
+        # type: (str, List[Tuple[str, str, List[str]]]) -> List[str]
         field_type = ':%s:' % field_type.strip()
         padding = ' ' * len(field_type)
         multi = len(fields) > 1
-        lines = []
+        lines = []  # type: List[str]
         for _name, _type, _desc in fields:
             field = self._format_field(_name, _type, _desc)
             if multi:
@@ -404,6 +444,7 @@ class GoogleDocstring(UnicodeMixin):
         return lines
 
     def _get_current_indent(self, peek_ahead=0):
+        # type: (int) -> int
         line = self._line_iter.peek(peek_ahead + 1)[peek_ahead]
         while line != self._line_iter.sentinel:
             if line:
@@ -413,18 +454,21 @@ class GoogleDocstring(UnicodeMixin):
         return 0
 
     def _get_indent(self, line):
+        # type: (str) -> int
         for i, s in enumerate(line):
             if not s.isspace():
                 return i
         return len(line)
 
     def _get_initial_indent(self, lines):
+        # type: (List[str]) -> int
         for line in lines:
             if line:
                 return self._get_indent(line)
         return 0
 
     def _get_min_indent(self, lines):
+        # type: (List[str]) -> int
         min_indent = None
         for line in lines:
             if line:
@@ -436,9 +480,11 @@ class GoogleDocstring(UnicodeMixin):
         return min_indent or 0
 
     def _indent(self, lines, n=4):
+        # type: (List[str], int) -> List[str]
         return [(' ' * n) + line for line in lines]
 
     def _is_indented(self, line, indent=1):
+        # type: (str, int) -> bool
         for i, s in enumerate(line):
             if i >= indent:
                 return True
@@ -447,6 +493,7 @@ class GoogleDocstring(UnicodeMixin):
         return False
 
     def _is_list(self, lines):
+        # type: (List[str]) -> bool
         if not lines:
             return False
         if _bullet_list_regex.match(lines[0]):
@@ -464,6 +511,7 @@ class GoogleDocstring(UnicodeMixin):
         return next_indent > indent
 
     def _is_section_header(self):
+        # type: () -> bool
         section = self._line_iter.peek().lower()
         match = _google_section_regex.match(section)
         if match and section.strip(':') in self._sections:
@@ -478,6 +526,7 @@ class GoogleDocstring(UnicodeMixin):
         return False
 
     def _is_section_break(self):
+        # type: () -> bool
         line = self._line_iter.peek()
         return (not self._line_iter.has_next() or
                 self._is_section_header() or
@@ -485,11 +534,36 @@ class GoogleDocstring(UnicodeMixin):
                     line and
                     not self._is_indented(line, self._section_indent)))
 
+    def _load_custom_sections(self):
+        # type: () -> None
+
+        if self._config.napoleon_custom_sections is not None:
+            for entry in self._config.napoleon_custom_sections:
+                if isinstance(entry, str):
+                    # if entry is just a label, add to sections list,
+                    # using generic section logic.
+                    self._sections[entry.lower()] = self._parse_custom_generic_section
+                else:
+                    # otherwise, assume entry is container;
+                    # [0] is new section, [1] is the section to alias.
+                    # in the case of key mismatch, just handle as generic section.
+                    self._sections[entry[0].lower()] = \
+                        self._sections.get(entry[1].lower(),
+                                           self._parse_custom_generic_section)
+
     def _parse(self):
+        # type: () -> None
         self._parsed_lines = self._consume_empty()
 
         if self._name and (self._what == 'attribute' or self._what == 'data'):
-            self._parsed_lines.extend(self._parse_attribute_docstring())
+            # Implicit stop using StopIteration no longer allowed in
+            # Python 3.7; see PEP 479
+            res = []  # type: List[str]
+            try:
+                res = self._parse_attribute_docstring()
+            except StopIteration:
+                pass
+            self._parsed_lines.extend(res)
             return
 
         while self._line_iter.has_next():
@@ -512,32 +586,57 @@ class GoogleDocstring(UnicodeMixin):
                     lines = self._consume_to_next_section()
             self._parsed_lines.extend(lines)
 
+    def _parse_admonition(self, admonition, section):
+        # type (str, str) -> List[str]
+        lines = self._consume_to_next_section()
+        return self._format_admonition(admonition, lines)
+
     def _parse_attribute_docstring(self):
+        # type: () -> List[str]
         _type, _desc = self._consume_inline_attribute()
-        return self._format_field('', _type, _desc)
+        lines = self._format_field('', '', _desc)
+        if _type:
+            lines.extend(['', ':type: %s' % _type])
+        return lines
 
     def _parse_attributes_section(self, section):
+        # type: (str) -> List[str]
         lines = []
         for _name, _type, _desc in self._consume_fields():
             if self._config.napoleon_use_ivar:
+                _name = self._qualify_name(_name, self._obj)
                 field = ':ivar %s: ' % _name
                 lines.extend(self._format_block(field, _desc))
                 if _type:
                     lines.append(':vartype %s: %s' % (_name, _type))
             else:
                 lines.extend(['.. attribute:: ' + _name, ''])
-                field = self._format_field('', _type, _desc)
-                lines.extend(self._indent(field, 3))
+                fields = self._format_field('', '', _desc)
+                lines.extend(self._indent(fields, 3))
+                if _type:
+                    lines.append('')
+                    lines.extend(self._indent([':type: %s' % _type], 3))
                 lines.append('')
         if self._config.napoleon_use_ivar:
             lines.append('')
         return lines
 
     def _parse_examples_section(self, section):
+        # type: (str) -> List[str]
+        labels = {
+            'example': _('Example'),
+            'examples': _('Examples'),
+        }
         use_admonition = self._config.napoleon_use_admonition_for_examples
-        return self._parse_generic_section(section, use_admonition)
+        label = labels.get(section.lower(), section)
+        return self._parse_generic_section(label, use_admonition)
+
+    def _parse_custom_generic_section(self, section):
+        # for now, no admonition for simple custom sections
+        return self._parse_generic_section(section, False)
 
     def _parse_usage_section(self, section):
+        # type: (str) -> List[str]
         header = ['.. rubric:: Usage:', '']
         block = ['.. code-block:: python', '']
         lines = self._consume_usage_section()
@@ -545,6 +644,7 @@ class GoogleDocstring(UnicodeMixin):
         return header + block + lines + ['']
 
     def _parse_generic_section(self, section, use_admonition):
+        # type: (str, bool) -> List[str]
         lines = self._strip_empty(self._consume_to_next_section())
         lines = self._dedent(lines)
         if use_admonition:
@@ -558,6 +658,7 @@ class GoogleDocstring(UnicodeMixin):
             return [header, '']
 
     def _parse_keyword_arguments_section(self, section):
+        # type: (str) -> List[str]
         fields = self._consume_fields()
         if self._config.napoleon_use_keyword:
             return self._format_docutils_params(
@@ -565,78 +666,58 @@ class GoogleDocstring(UnicodeMixin):
                 field_role="keyword",
                 type_role="kwtype")
         else:
-            return self._format_fields('Keyword Arguments', fields)
+            return self._format_fields(_('Keyword Arguments'), fields)
 
     def _parse_methods_section(self, section):
-        lines = []
-        for _name, _, _desc in self._consume_fields(parse_type=False):
+        # type: (str) -> List[str]
+        lines = []  # type: List[str]
+        for _name, _type, _desc in self._consume_fields(parse_type=False):
             lines.append('.. method:: %s' % _name)
             if _desc:
                 lines.extend([''] + self._indent(_desc, 3))
             lines.append('')
         return lines
 
-    def _parse_note_section(self, section):
-        lines = self._consume_to_next_section()
-        return self._format_admonition('note', lines)
-
     def _parse_notes_section(self, section):
+        # type: (str) -> List[str]
         use_admonition = self._config.napoleon_use_admonition_for_notes
-        return self._parse_generic_section('Notes', use_admonition)
+        return self._parse_generic_section(_('Notes'), use_admonition)
 
     def _parse_other_parameters_section(self, section):
-        return self._format_fields('Other Parameters', self._consume_fields())
+        # type: (str) -> List[str]
+        return self._format_fields(_('Other Parameters'), self._consume_fields())
 
     def _parse_parameters_section(self, section):
+        # type: (str) -> List[str]
         fields = self._consume_fields()
         if self._config.napoleon_use_param:
             return self._format_docutils_params(fields)
         else:
-            return self._format_fields('Parameters', fields)
+            return self._format_fields(_('Parameters'), fields)
 
     def _parse_raises_section(self, section):
+        # type: (str) -> List[str]
         fields = self._consume_fields(parse_type=False, prefer_type=True)
-        field_type = ':raises:'
-        padding = ' ' * len(field_type)
-        multi = len(fields) > 1
-        lines = []
-        for _, _type, _desc in fields:
+        lines = []  # type: List[str]
+        for _name, _type, _desc in fields:
+            m = self._name_rgx.match(_type)
+            if m and m.group('name'):
+                _type = m.group('name')
+            _type = ' ' + _type if _type else ''
             _desc = self._strip_empty(_desc)
-            has_desc = any(_desc)
-            separator = has_desc and ' -- ' or ''
-            if _type:
-                has_refs = '`' in _type or ':' in _type
-                has_space = any(c in ' \t\n\v\f ' for c in _type)
-
-                if not has_refs and not has_space:
-                    _type = ':exc:`%s`%s' % (_type, separator)
-                elif has_desc and has_space:
-                    _type = '*%s*%s' % (_type, separator)
-                else:
-                    _type = '%s%s' % (_type, separator)
-
-                if has_desc:
-                    field = [_type + _desc[0]] + _desc[1:]
-                else:
-                    field = [_type]
-            else:
-                field = _desc
-            if multi:
-                if lines:
-                    lines.extend(self._format_block(padding + ' * ', field))
-                else:
-                    lines.extend(self._format_block(field_type + ' * ', field))
-            else:
-                lines.extend(self._format_block(field_type + ' ', field))
-        if lines and lines[-1]:
+            _descs = ' ' + '\n    '.join(_desc) if any(_desc) else ''
+            lines.append(':raises%s:%s' % (_type, _descs))
+        if lines:
             lines.append('')
         return lines
 
     def _parse_references_section(self, section):
+        # type: (str) -> List[str]
         use_admonition = self._config.napoleon_use_admonition_for_references
-        return self._parse_generic_section('References', use_admonition)
+        return self._parse_generic_section(_('References'), use_admonition)
 
     def _parse_returns_section(self, section):
+        # type: (str) -> List[str]
         fields = self._consume_returns_section()
         multi = len(fields) > 1
         if multi:
@@ -644,7 +725,7 @@ class GoogleDocstring(UnicodeMixin):
         else:
             use_rtype = self._config.napoleon_use_rtype
 
-        lines = []
+        lines = []  # type: List[str]
         for _name, _type, _desc in fields:
             if use_rtype:
                 field = self._format_field(_name, '', _desc)
@@ -665,25 +746,20 @@ class GoogleDocstring(UnicodeMixin):
         return lines
 
     def _parse_see_also_section(self, section):
-        lines = self._consume_to_next_section()
-        return self._format_admonition('seealso', lines)
-
-    def _parse_todo_section(self, section):
-        lines = self._consume_to_next_section()
-        return self._format_admonition('todo', lines)
-
-    def _parse_warning_section(self, section):
-        lines = self._consume_to_next_section()
-        return self._format_admonition('warning', lines)
+        # type (str) -> List[str]
+        return self._parse_admonition('seealso', section)
 
     def _parse_warns_section(self, section):
-        return self._format_fields('Warns', self._consume_fields())
+        # type: (str) -> List[str]
+        return self._format_fields(_('Warns'), self._consume_fields())
 
     def _parse_yields_section(self, section):
+        # type: (str) -> List[str]
         fields = self._consume_returns_section()
-        return self._format_fields('Yields', fields)
+        return self._format_fields(_('Yields'), fields)
 
     def _partition_field_on_colon(self, line):
+        # type: (str) -> Tuple[str, str, str]
         before_colon = []
         after_colon = []
         colon = ''
@@ -705,7 +781,20 @@ class GoogleDocstring(UnicodeMixin):
                 colon,
                 "".join(after_colon).strip())
 
+    def _qualify_name(self, attr_name, klass):
+        # type: (str, Type) -> str
+        if klass and '.' not in attr_name:
+            if attr_name.startswith('~'):
+                attr_name = attr_name[1:]
+            try:
+                q = klass.__qualname__
+            except AttributeError:
+                q = klass.__name__
+            return '~%s.%s' % (q, attr_name)
+        return attr_name
+
     def _strip_empty(self, lines):
+        # type: (List[str]) -> List[str]
         if lines:
             start = -1
             for i, line in enumerate(lines):
@@ -820,11 +909,12 @@ class NumpyDocstring(GoogleDocstring):
     """
     def __init__(self, docstring, config=None, app=None, what='', name='',
                  obj=None, options=None):
+        # type: (Union[str, List[str]], SphinxConfig, Sphinx, str, str, Any, Any) -> None
         self._directive_sections = ['.. index::']
-        super(NumpyDocstring, self).__init__(docstring, config, app, what,
-                                             name, obj, options)
+        super().__init__(docstring, config, app, what, name, obj, options)
 
     def _consume_field(self, parse_type=True, prefer_type=False):
+        # type: (bool, bool) -> Tuple[str, str, List[str]]
         line = next(self._line_iter)
         if parse_type:
             _name, _, _type = self._partition_field_on_colon(line)
@@ -841,9 +931,11 @@ class NumpyDocstring(GoogleDocstring):
         return _name, _type, _desc
 
     def _consume_returns_section(self):
+        # type: () -> List[Tuple[str, str, List[str]]]
         return self._consume_fields(prefer_type=True)
 
     def _consume_section_header(self):
+        # type: () -> str
         section = next(self._line_iter)
         if not _directive_regex.match(section):
             # Consume the header underline
@@ -851,6 +943,7 @@ class NumpyDocstring(GoogleDocstring):
         return section
 
     def _is_section_break(self):
+        # type: () -> bool
         line1, line2 = self._line_iter.peek(2)
         return (not self._line_iter.has_next() or
                 self._is_section_header() or
@@ -860,9 +953,10 @@ class NumpyDocstring(GoogleDocstring):
                     not self._is_indented(line1, self._section_indent)))
 
     def _is_section_header(self):
+        # type: () -> bool
         section, underline = self._line_iter.peek(2)
         section = section.lower()
-        if section in self._sections and isinstance(underline, string_types):
+        if section in self._sections and isinstance(underline, str):
             return bool(_numpy_section_regex.match(underline))
         elif self._directive_sections:
             if _directive_regex.match(section):
@@ -871,10 +965,8 @@ class NumpyDocstring(GoogleDocstring):
                         return True
         return False
 
-    _name_rgx = re.compile(r"^\s*(:(?P<role>\w+):`(?P<name>[a-zA-Z0-9_.-]+)`|"
-                           r" (?P<name2>[a-zA-Z0-9_.-]+))\s*", re.X)
-
     def _parse_see_also_section(self, section):
+        # type: (str) -> List[str]
         lines = self._consume_to_next_section()
         try:
             return self._parse_numpydoc_see_also_section(lines)
@@ -882,6 +974,7 @@ class NumpyDocstring(GoogleDocstring):
             return self._format_admonition('seealso', lines)
 
     def _parse_numpydoc_see_also_section(self, content):
+        # type: (List[str]) -> List[str]
         """
         Derived from the NumpyDoc implementation of _parse_see_also.
 
@@ -896,6 +989,7 @@ class NumpyDocstring(GoogleDocstring):
         items = []
 
         def parse_item_name(text):
+            # type: (str) -> Tuple[str, str]
             """Match ':role:`name`' or 'name'"""
             m = self._name_rgx.match(text)
             if m:
@@ -907,6 +1001,7 @@ class NumpyDocstring(GoogleDocstring):
             raise ValueError("%s is not a item name" % text)
 
         def push_item(name, rest):
+            # type: (str, List[str]) -> None
             if not name:
                 return
             name, role = parse_item_name(name)
@@ -914,7 +1009,7 @@ class NumpyDocstring(GoogleDocstring):
             del rest[:]
 
         current_func = None
-        rest = []
+        rest = []  # type: List[str]
 
         for line in content:
             if not line.strip():
@@ -965,7 +1060,7 @@ class NumpyDocstring(GoogleDocstring):
             func_role = 'obj'
         else:
             func_role = roles.get(self._what, '')
-        lines = []
+        lines = []  # type: List[str]
         last_had_desc = True
         for func, desc, role in items:
             if role:
