@@ -1,39 +1,39 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.pycode
     ~~~~~~~~~~~~~
 
     Utilities parsing and analyzing Python code.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
-from __future__ import print_function
 
-from typing import TYPE_CHECKING
-
-from six import iteritems, BytesIO, StringIO
+import re
+from io import StringIO
+from os import path
+from zipfile import ZipFile
 
 from sphinx.errors import PycodeError
 from sphinx.pycode.parser import Parser
 from sphinx.util import get_module_source, detect_encoding
 
-if TYPE_CHECKING:
+if False:
+    # For type annotation
     from typing import Any, Dict, IO, List, Tuple  # NOQA
 
 
-class ModuleAnalyzer(object):
+class ModuleAnalyzer:
     # cache for analyzer objects -- caches both by module and file name
-    cache = {}  # type: Dict[Tuple[unicode, unicode], Any]
+    cache = {}  # type: Dict[Tuple[str, str], Any]
 
     @classmethod
     def for_string(cls, string, modname, srcname='<string>'):
-        if isinstance(string, bytes):
-            return cls(BytesIO(string), modname, srcname)
+        # type: (str, str, str) -> ModuleAnalyzer
         return cls(StringIO(string), modname, srcname, decoded=True)
 
     @classmethod
     def for_file(cls, filename, modname):
+        # type: (str, str) -> ModuleAnalyzer
         if ('file', filename) in cls.cache:
             return cls.cache['file', filename]
         try:
@@ -41,11 +41,27 @@ class ModuleAnalyzer(object):
                 obj = cls(f, modname, filename)
                 cls.cache['file', filename] = obj
         except Exception as err:
-            raise PycodeError('error opening %r' % filename, err)
+            if '.egg' + path.sep in filename:
+                obj = cls.cache['file', filename] = cls.for_egg(filename, modname)
+            else:
+                raise PycodeError('error opening %r' % filename, err)
         return obj
 
+    @classmethod
+    def for_egg(cls, filename, modname):
+        # type: (str, str) -> ModuleAnalyzer
+        SEP = re.escape(path.sep)
+        eggpath, relpath = re.split('(?<=\\.egg)' + SEP, filename)
+        try:
+            with ZipFile(eggpath) as egg:
+                code = egg.read(relpath).decode()
+                return cls.for_string(code, modname, filename)
+        except Exception as exc:
+            raise PycodeError('error opening %r' % filename, exc)
+
     @classmethod
     def for_module(cls, modname):
+        # type: (str) -> ModuleAnalyzer
         if ('module', modname) in cls.cache:
             entry = cls.cache['module', modname]
             if isinstance(entry, PycodeError):
@@ -65,7 +81,7 @@ class ModuleAnalyzer(object):
         return obj
 
     def __init__(self, source, modname, srcname, decoded=False):
-        # type: (IO, unicode, unicode, bool) -> None
+        # type: (IO, str, str, bool) -> None
         self.modname = modname  # name of the module
         self.srcname = srcname  # name of the source file
 
@@ -80,9 +96,9 @@ class ModuleAnalyzer(object):
             self.code = source.read()
 
         # will be filled by parse()
-        self.attr_docs = None   # type: Dict[Tuple[unicode, unicode], List[unicode]]
-        self.tagorder = None    # type: Dict[unicode, int]
-        self.tags = None        # type: Dict[unicode, Tuple[unicode, int, int]]
+        self.attr_docs = None   # type: Dict[Tuple[str, str], List[str]]
+        self.tagorder = None    # type: Dict[str, int]
+        self.tags = None        # type: Dict[str, Tuple[str, int, int]]
 
     def parse(self):
         # type: () -> None
@@ -92,7 +108,7 @@ class ModuleAnalyzer(object):
             parser.parse()
 
             self.attr_docs = {}
-            for (scope, comment) in iteritems(parser.comments):
+            for (scope, comment) in parser.comments.items():
                 if comment:
                     self.attr_docs[scope] = comment.splitlines() + ['']
                 else:
@@ -104,7 +120,7 @@ class ModuleAnalyzer(object):
             raise PycodeError('parsing %r failed: %r' % (self.srcname, exc))
 
     def find_attr_docs(self):
-        # type: () -> Dict[Tuple[unicode, unicode], List[unicode]]
+        # type: () -> Dict[Tuple[str, str], List[str]]
         """Find class and module-level attributes and their documentation."""
         if self.attr_docs is None:
             self.parse()
@@ -112,29 +128,9 @@ class ModuleAnalyzer(object):
         return self.attr_docs
 
     def find_tags(self):
-        # type: () -> Dict[unicode, Tuple[unicode, int, int]]
+        # type: () -> Dict[str, Tuple[str, int, int]]
         """Find class, function and method definitions and their location."""
         if self.tags is None:
             self.parse()
 
         return self.tags
-
-
-if __name__ == '__main__':
-    import time
-    import pprint
-    x0 = time.time()
-    # ma = ModuleAnalyzer.for_file(__file__.rstrip('c'), 'sphinx.builders.html')
-    ma = ModuleAnalyzer.for_file('sphinx/environment.py',
-                                 'sphinx.environment')
-    ma.tokenize()
-    x1 = time.time()
-    ma.parse()
-    x2 = time.time()
-    # for (ns, name), doc in iteritems(ma.find_attr_docs()):
-    #     print '>>', ns, name
-    #     print '\n'.join(doc)
-    pprint.pprint(ma.find_tags())
-    x3 = time.time()
-    # print nodes.nice_repr(ma.parsetree, number2name)
-    print("tokenizing %.4f, parsing %.4f, finding %.4f" % (x1 - x0, x2 - x1, x3 - x2))
