@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.search
     ~~~~~~~~~~~~~
@@ -8,18 +7,19 @@
     :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
+import html
+import pickle
 import re
+import warnings
 from os import path
 
-from six import iteritems, itervalues, text_type, string_types
-from six.moves import cPickle as pickle
+from docutils import nodes
 
-from docutils.nodes import raw, comment, title, Text, NodeVisitor, SkipNode
-
-import sphinx
-from sphinx.util import jsdump, rpartition
-from sphinx.util.pycompat import htmlescape
+from sphinx import addnodes
+from sphinx import package_dir
+from sphinx.deprecation import RemovedInSphinx40Warning
 from sphinx.search.jssplitter import splitter_code
+from sphinx.util import jsdump, rpartition
 
 if False:
     # For type annotation
@@ -28,7 +28,7 @@ if False:
     from sphinx.environment import BuildEnvironment  # NOQA
 
 
-class SearchLanguage(object):
+class SearchLanguage:
     """
     This class is the base class for search natural language preprocessors.  If
     you want to add support for a new language, you should override the methods
@@ -50,10 +50,10 @@ class SearchLanguage(object):
        This class is used to preprocess search word which Sphinx HTML readers
        type, before searching index. Default implementation does nothing.
     """
-    lang = None                 # type: unicode
-    language_name = None        # type: unicode
-    stopwords = set()           # type: Set[unicode]
-    js_stemmer_rawcode = None   # type: unicode
+    lang = None                 # type: str
+    language_name = None        # type: str
+    stopwords = set()           # type: Set[str]
+    js_stemmer_rawcode = None   # type: str
     js_stemmer_code = """
 /**
  * Dummy stemmer for languages without stemming rules.
@@ -63,7 +63,7 @@ var Stemmer = function() {
     return w;
   }
 }
-"""                             # type: unicode
+"""
 
     _word_re = re.compile(r'(?u)\w+')
 
@@ -79,7 +79,7 @@ var Stemmer = function() {
         """
 
     def split(self, input):
-        # type: (unicode) -> List[unicode]
+        # type: (str) -> List[str]
         """
         This method splits a sentence into words.  Default splitter splits input
         at white spaces, which should be enough for most languages except CJK
@@ -88,7 +88,7 @@ var Stemmer = function() {
         return self._word_re.findall(input)
 
     def stem(self, word):
-        # type: (unicode) -> unicode
+        # type: (str) -> str
         """
         This method implements stemming algorithm of the Python version.
 
@@ -102,7 +102,7 @@ var Stemmer = function() {
         return word
 
     def word_filter(self, word):
-        # type: (unicode) -> bool
+        # type: (str) -> bool
         """
         Return true if the target word should be registered in the search index.
         This method is called after stemming.
@@ -120,13 +120,13 @@ from sphinx.search.en import SearchEnglish
 
 
 def parse_stop_word(source):
-    # type: (unicode) -> Set[unicode]
+    # type: (str) -> Set[str]
     """
     parse snowball style word list like this:
 
     * http://snowball.tartarus.org/algorithms/finnish/stop.txt
     """
-    result = set()
+    result = set()  # type: Set[str]
     for line in source.splitlines():
         line = line.split('|')[0]  # remove comment
         result.update(line.split())
@@ -152,10 +152,10 @@ languages = {
     'sv': 'sphinx.search.sv.SearchSwedish',
     'tr': 'sphinx.search.tr.SearchTurkish',
     'zh': 'sphinx.search.zh.SearchChinese',
-}   # type: Dict[unicode, Any]
+}  # type: Dict[str, Any]
 
 
-class _JavaScriptIndex(object):
+class _JavaScriptIndex:
     """
     The search index as javascript file that calls a function
     on the documentation search object to register the index.
@@ -165,7 +165,7 @@ class _JavaScriptIndex(object):
     SUFFIX = ')'
 
     def dumps(self, data):
-        # type: (Any) -> unicode
+        # type: (Any) -> str
         return self.PREFIX + jsdump.dumps(data) + self.SUFFIX
 
     def loads(self, s):
@@ -188,21 +188,25 @@ class _JavaScriptIndex(object):
 js_index = _JavaScriptIndex()
 
 
-class WordCollector(NodeVisitor):
+class WordCollector(nodes.NodeVisitor):
     """
     A special visitor that collects words for the `IndexBuilder`.
     """
 
     def __init__(self, document, lang):
-        # type: (nodes.Node, SearchLanguage) -> None
-        NodeVisitor.__init__(self, document)
-        self.found_words = []           # type: List[unicode]
-        self.found_title_words = []     # type: List[unicode]
+        # type: (nodes.document, SearchLanguage) -> None
+        super().__init__(document)
+        self.found_words = []           # type: List[str]
+        self.found_title_words = []     # type: List[str]
         self.lang = lang
 
-    def is_meta_keywords(self, node, nodetype):
-        # type: (nodes.Node, Type) -> bool
-        if isinstance(node, sphinx.addnodes.meta) and node.get('name') == 'keywords':
+    def is_meta_keywords(self, node, nodetype=None):
+        # type: (addnodes.meta, Any) -> bool
+        if nodetype is not None:
+            warnings.warn('"nodetype" argument for WordCollector.is_meta_keywords() '
+                          'is deprecated.', RemovedInSphinx40Warning)
+
+        if isinstance(node, addnodes.meta) and node.get('name') == 'keywords':
             meta_lang = node.get('lang')
             if meta_lang is None:  # lang not specified
                 return True
@@ -213,10 +217,9 @@ class WordCollector(NodeVisitor):
 
     def dispatch_visit(self, node):
         # type: (nodes.Node) -> None
-        nodetype = type(node)
-        if issubclass(nodetype, comment):
-            raise SkipNode
-        if issubclass(nodetype, raw):
+        if isinstance(node, nodes.comment):
+            raise nodes.SkipNode
+        elif isinstance(node, nodes.raw):
             if 'html' in node.get('format', '').split():
                 # Some people might put content in raw HTML that should be searched,
                 # so we just amateurishly strip HTML tags and index the remaining
@@ -225,18 +228,18 @@ class WordCollector(NodeVisitor):
                 nodetext = re.sub(r'(?is)<script.*?</script>', '', nodetext)
                 nodetext = re.sub(r'<[^<]+?>', '', nodetext)
                 self.found_words.extend(self.lang.split(nodetext))
-            raise SkipNode
-        if issubclass(nodetype, Text):
+            raise nodes.SkipNode
+        elif isinstance(node, nodes.Text):
             self.found_words.extend(self.lang.split(node.astext()))
-        elif issubclass(nodetype, title):
+        elif isinstance(node, nodes.title):
             self.found_title_words.extend(self.lang.split(node.astext()))
-        elif self.is_meta_keywords(node, nodetype):
+        elif isinstance(node, addnodes.meta) and self.is_meta_keywords(node):
             keywords = node['content']
             keywords = [keyword.strip() for keyword in keywords.split(',')]
             self.found_words.extend(keywords)
 
 
-class IndexBuilder(object):
+class IndexBuilder:
     """
     Helper class that creates a searchindex based on the doctrees
     passed to the `feed` method.
@@ -244,24 +247,24 @@ class IndexBuilder(object):
     formats = {
         'jsdump':   jsdump,
         'pickle':   pickle
-    }   # type: Dict[unicode, Any]
+    }
 
     def __init__(self, env, lang, options, scoring):
-        # type: (BuildEnvironment, unicode, Dict, unicode) -> None
+        # type: (BuildEnvironment, str, Dict, str) -> None
         self.env = env
-        self._titles = {}           # type: Dict[unicode, unicode]
+        self._titles = {}           # type: Dict[str, str]
                                     # docname -> title
-        self._filenames = {}        # type: Dict[unicode, unicode]
+        self._filenames = {}        # type: Dict[str, str]
                                     # docname -> filename
-        self._mapping = {}          # type: Dict[unicode, Set[unicode]]
+        self._mapping = {}          # type: Dict[str, Set[str]]
                                     # stemmed word -> set(docname)
-        self._title_mapping = {}    # type: Dict[unicode, Set[unicode]]
+        self._title_mapping = {}    # type: Dict[str, Set[str]]
                                     # stemmed words in titles -> set(docname)
-        self._stem_cache = {}       # type: Dict[unicode, unicode]
+        self._stem_cache = {}       # type: Dict[str, str]
                                     # word -> stemmed word
-        self._objtypes = {}         # type: Dict[Tuple[unicode, unicode], int]
+        self._objtypes = {}         # type: Dict[Tuple[str, str], int]
                                     # objtype -> index
-        self._objnames = {}         # type: Dict[int, Tuple[unicode, unicode, unicode]]
+        self._objnames = {}         # type: Dict[int, Tuple[str, str, str]]
                                     # objtype index -> (domain, type, objname (localized))
         lang_class = languages.get(lang)    # type: Type[SearchLanguage]
                                             # add language-specific SearchLanguage instance
@@ -283,15 +286,15 @@ class IndexBuilder(object):
 
         if scoring:
             with open(scoring, 'rb') as fp:
-                self.js_scorer_code = fp.read().decode('utf-8')
+                self.js_scorer_code = fp.read().decode()
         else:
-            self.js_scorer_code = u''
+            self.js_scorer_code = ''
         self.js_splitter_code = splitter_code
 
     def load(self, stream, format):
         # type: (IO, Any) -> None
         """Reconstruct from frozen data."""
-        if isinstance(format, string_types):
+        if isinstance(format, str):
             format = self.formats[format]
         frozen = format.load(stream)
         # if an old index is present, we treat it as not existing.
@@ -303,13 +306,13 @@ class IndexBuilder(object):
         self._titles = dict(zip(index2fn, frozen['titles']))
 
         def load_terms(mapping):
-            # type: (Dict[unicode, Any]) -> Dict[unicode, Set[unicode]]
+            # type: (Dict[str, Any]) -> Dict[str, Set[str]]
             rv = {}
-            for k, v in iteritems(mapping):
+            for k, v in mapping.items():
                 if isinstance(v, int):
-                    rv[k] = set([index2fn[v]])
+                    rv[k] = {index2fn[v]}
                 else:
-                    rv[k] = set(index2fn[i] for i in v)
+                    rv[k] = {index2fn[i] for i in v}
             return rv
 
         self._mapping = load_terms(frozen['terms'])
@@ -319,24 +322,24 @@ class IndexBuilder(object):
     def dump(self, stream, format):
         # type: (IO, Any) -> None
         """Dump the frozen index to a stream."""
-        if isinstance(format, string_types):
+        if isinstance(format, str):
             format = self.formats[format]
         format.dump(self.freeze(), stream)
 
     def get_objects(self, fn2index):
-        # type: (Dict[unicode, int]) -> Dict[unicode, Dict[unicode, Tuple[int, int, int, unicode]]]  # NOQA
-        rv = {}  # type: Dict[unicode, Dict[unicode, Tuple[int, int, int, unicode]]]
+        # type: (Dict[str, int]) -> Dict[str, Dict[str, Tuple[int, int, int, str]]]
+        rv = {}  # type: Dict[str, Dict[str, Tuple[int, int, int, str]]]
         otypes = self._objtypes
         onames = self._objnames
-        for domainname, domain in sorted(iteritems(self.env.domains)):
+        for domainname, domain in sorted(self.env.domains.items()):
             for fullname, dispname, type, docname, anchor, prio in \
                     sorted(domain.get_objects()):
                 if docname not in fn2index:
                     continue
                 if prio < 0:
                     continue
-                fullname = htmlescape(fullname)
-                dispname = htmlescape(dispname)
+                fullname = html.escape(fullname)
+                dispname = html.escape(dispname)
                 prefix, name = rpartition(dispname, '.')
                 pdict = rv.setdefault(prefix, {})
                 try:
@@ -346,13 +349,13 @@ class IndexBuilder(object):
                     otypes[domainname, type] = typeindex
                     otype = domain.object_types.get(type)
                     if otype:
-                        # use unicode() to fire translation proxies
+                        # use str() to fire translation proxies
                         onames[typeindex] = (domainname, type,
-                                             text_type(domain.get_type_name(otype)))
+                                             str(domain.get_type_name(otype)))
                     else:
                         onames[typeindex] = (domainname, type, type)
                 if anchor == fullname:
-                    shortanchor = ''  # type: unicode
+                    shortanchor = ''
                 elif anchor == type + '-' + fullname:
                     shortanchor = '-'
                 else:
@@ -361,10 +364,10 @@ class IndexBuilder(object):
         return rv
 
     def get_terms(self, fn2index):
-        # type: (Dict) -> Tuple[Dict[unicode, List[unicode]], Dict[unicode, List[unicode]]]
-        rvs = {}, {}  # type: Tuple[Dict[unicode, List[unicode]], Dict[unicode, List[unicode]]]
+        # type: (Dict) -> Tuple[Dict[str, List[str]], Dict[str, List[str]]]
+        rvs = {}, {}  # type: Tuple[Dict[str, List[str]], Dict[str, List[str]]]
         for rv, mapping in zip(rvs, (self._mapping, self._title_mapping)):
-            for k, v in iteritems(mapping):
+            for k, v in mapping.items():
                 if len(v) == 1:
                     fn, = v
                     if fn in fn2index:
@@ -374,27 +377,26 @@ class IndexBuilder(object):
         return rvs
 
     def freeze(self):
-        # type: () -> Dict[unicode, Any]
+        # type: () -> Dict[str, Any]
         """Create a usable data structure for serializing."""
         docnames, titles = zip(*sorted(self._titles.items()))
         filenames = [self._filenames.get(docname) for docname in docnames]
-        fn2index = dict((f, i) for (i, f) in enumerate(docnames))
+        fn2index = {f: i for (i, f) in enumerate(docnames)}
         terms, title_terms = self.get_terms(fn2index)
 
         objects = self.get_objects(fn2index)  # populates _objtypes
-        objtypes = dict((v, k[0] + ':' + k[1])
-                        for (k, v) in iteritems(self._objtypes))
+        objtypes = {v: k[0] + ':' + k[1] for (k, v) in self._objtypes.items()}
         objnames = self._objnames
         return dict(docnames=docnames, filenames=filenames, titles=titles, terms=terms,
                     objects=objects, objtypes=objtypes, objnames=objnames,
                     titleterms=title_terms, envversion=self.env.version)
 
     def label(self):
-        # type: () -> unicode
+        # type: () -> str
         return "%s (code: %s)" % (self.lang.language_name, self.lang.lang)
 
     def prune(self, docnames):
-        # type: (Iterable[unicode]) -> None
+        # type: (Iterable[str]) -> None
         """Remove data for all docnames not in the list."""
         new_titles = {}
         new_filenames = {}
@@ -404,13 +406,13 @@ class IndexBuilder(object):
                 new_filenames[docname] = self._filenames[docname]
         self._titles = new_titles
         self._filenames = new_filenames
-        for wordnames in itervalues(self._mapping):
+        for wordnames in self._mapping.values():
             wordnames.intersection_update(docnames)
-        for wordnames in itervalues(self._title_mapping):
+        for wordnames in self._title_mapping.values():
             wordnames.intersection_update(docnames)
 
     def feed(self, docname, filename, title, doctree):
-        # type: (unicode, unicode, unicode, nodes.Node) -> None
+        # type: (str, str, str, nodes.document) -> None
         """Feed a doctree to the index."""
         self._titles[docname] = title
         self._filenames[docname] = filename
@@ -420,7 +422,7 @@ class IndexBuilder(object):
 
         # memoize self.lang.stem
         def stem(word):
-            # type: (unicode) -> unicode
+            # type: (str) -> str
             try:
                 return self._stem_cache[word]
             except KeyError:
@@ -445,21 +447,18 @@ class IndexBuilder(object):
                 self._mapping.setdefault(stemmed_word, set()).add(docname)
 
     def context_for_searchtool(self):
-        # type: () -> Dict[unicode, Any]
-        return dict(
-            search_language_stemming_code = self.lang.js_stemmer_code,
-            search_language_stop_words = jsdump.dumps(sorted(self.lang.stopwords)),
-            search_scorer_tool = self.js_scorer_code,
-            search_word_splitter_code = self.js_splitter_code,
-        )
+        # type: () -> Dict[str, Any]
+        return {
+            'search_language_stemming_code': self.lang.js_stemmer_code,
+            'search_language_stop_words': jsdump.dumps(sorted(self.lang.stopwords)),
+            'search_scorer_tool': self.js_scorer_code,
+            'search_word_splitter_code': self.js_splitter_code,
+        }
 
     def get_js_stemmer_rawcode(self):
-        # type: () -> unicode
+        # type: () -> str
         if self.lang.js_stemmer_rawcode:
-            return path.join(
-                sphinx.package_dir, 'search',
-                'non-minified-js',
-                self.lang.js_stemmer_rawcode
-            )
+            return path.join(package_dir, 'search', 'non-minified-js',
+                             self.lang.js_stemmer_rawcode)
         else:
             return None
