@@ -1,19 +1,24 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.transforms.compact_bullet_list
     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
     Docutils transforms used by Sphinx when reading documents.
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
+from typing import cast
+
 from docutils import nodes
 
 from sphinx import addnodes
 from sphinx.transforms import SphinxTransform
 
+if False:
+    # For type annotation
+    from typing import Any, List  # NOQA
+
 
 class RefOnlyListChecker(nodes.GenericNodeVisitor):
     """Raise `nodes.NodeFound` if non-simple list item is encountered.
@@ -27,12 +32,12 @@ class RefOnlyListChecker(nodes.GenericNodeVisitor):
         raise nodes.NodeFound
 
     def visit_bullet_list(self, node):
-        # type: (nodes.Node) -> None
+        # type: (nodes.bullet_list) -> None
         pass
 
     def visit_list_item(self, node):
-        # type: (nodes.Node) -> None
-        children = []
+        # type: (nodes.list_item) -> None
+        children = []  # type: List[nodes.Node]
         for child in node.children:
             if not isinstance(child, nodes.Invisible):
                 children.append(child)
@@ -61,8 +66,8 @@ class RefOnlyBulletListTransform(SphinxTransform):
     """
     default_priority = 100
 
-    def apply(self):
-        # type: () -> None
+    def apply(self, **kwargs):
+        # type: (Any) -> None
         if self.config.html_compact_lists:
             return
 
@@ -80,8 +85,8 @@ class RefOnlyBulletListTransform(SphinxTransform):
         for node in self.document.traverse(nodes.bullet_list):
             if check_refonly_list(node):
                 for item in node.traverse(nodes.list_item):
-                    para = item[0]
-                    ref = para[0]
+                    para = cast(nodes.paragraph, item[0])
+                    ref = cast(nodes.reference, para[0])
                     compact_para = addnodes.compact_paragraph()
                     compact_para += ref
                     item.replace(para, compact_para)
