@@ -1,24 +1,22 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.util.docstrings
     ~~~~~~~~~~~~~~~~~~~~~~
 
     Utilities for docstring processing.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
-from __future__ import absolute_import
 
 import sys
-from typing import TYPE_CHECKING
 
-if TYPE_CHECKING:
+if False:
+    # For type annotation
     from typing import List  # NOQA
 
 
 def prepare_docstring(s, ignore=1):
-    # type: (unicode, int) -> List[unicode]
+    # type: (str, int) -> List[str]
     """Convert a docstring into lines of parseable reST.  Remove common leading
     indentation, where the indentation of a given number of lines (usually just
     one) is ignored.
@@ -52,7 +50,7 @@ def prepare_docstring(s, ignore=1):
 
 
 def prepare_commentdoc(s):
-    # type: (unicode) -> List[unicode]
+    # type: (str) -> List[str]
     """Extract documentation comment lines (starting with #:) and return them
     as a list of lines.  Returns an empty list if there is no documentation.
     """
