@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.util.jsdump
     ~~~~~~~~~~~~~~~~~~
@@ -6,15 +5,15 @@
     This module implements a simple JavaScript serializer.
     Uses the basestring encode function from simplejson by Bob Ippolito.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import re
 
-from six import iteritems, integer_types, string_types
-
-from sphinx.util.pycompat import u
+if False:
+    # For type annotation
+    from typing import Any, Dict, IO, List, Match, Union  # NOQA
 
 _str_re = re.compile(r'"(\\\\|\\"|[^"])*"')
 _int_re = re.compile(r'\d+')
@@ -37,7 +36,9 @@ ESCAPED = re.compile(r'\\u.{4}|\\.')
 
 
 def encode_string(s):
+    # type: (str) -> str
     def replace(match):
+        # type: (Match) -> str
         s = match.group(0)
         try:
             return ESCAPE_DICT[s]
@@ -55,7 +56,8 @@ def encode_string(s):
 
 
 def decode_string(s):
-    return ESCAPED.sub(lambda m: eval(u + '"' + m.group() + '"'), s)
+    # type: (str) -> str
+    return ESCAPED.sub(lambda m: eval('"' + m.group() + '"'), s)
 
 
 reswords = set("""\
@@ -77,8 +79,9 @@ double   in   super""".split())
 
 
 def dumps(obj, key=False):
+    # type: (Any, bool) -> str
     if key:
-        if not isinstance(obj, string_types):
+        if not isinstance(obj, str):
             obj = str(obj)
         if _nameonly_re.match(obj) and obj not in reswords:
             return obj  # return it as a bare word
@@ -88,33 +91,35 @@ def dumps(obj, key=False):
         return 'null'
     elif obj is True or obj is False:
         return obj and 'true' or 'false'
-    elif isinstance(obj, integer_types + (float,)):
+    elif isinstance(obj, (int, float)):
         return str(obj)
     elif isinstance(obj, dict):
         return '{%s}' % ','.join(sorted('%s:%s' % (
             dumps(key, True),
             dumps(value)
-        ) for key, value in iteritems(obj)))
+        ) for key, value in obj.items()))
     elif isinstance(obj, set):
         return '[%s]' % ','.join(sorted(dumps(x) for x in obj))
     elif isinstance(obj, (tuple, list)):
         return '[%s]' % ','.join(dumps(x) for x in obj)
-    elif isinstance(obj, string_types):
+    elif isinstance(obj, str):
         return encode_string(obj)
     raise TypeError(type(obj))
 
 
 def dump(obj, f):
+    # type: (Any, IO) -> None
     f.write(dumps(obj))
 
 
 def loads(x):
+    # type: (str) -> Any
     """Loader that can read the JS subset the indexer produces."""
     nothing = object()
     i = 0
     n = len(x)
-    stack = []
-    obj = nothing
+    stack = []  # type: List[Union[List, Dict]]
+    obj = nothing  # type: Any
     key = False
     keys = []
     while i < n:
@@ -164,6 +169,7 @@ def loads(x):
                 raise ValueError("multiple values")
             key = False
         else:
+            y = None  # type: Any
             m = _str_re.match(x, i)
             if m:
                 y = decode_string(m.group()[1:-1])
@@ -200,4 +206,5 @@ def loads(x):
 
 
 def load(f):
+    # type: (IO) -> Any
     return loads(f.read())
