@@ -1,24 +1,20 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.util.requests
     ~~~~~~~~~~~~~~~~~~~~
 
     Simple requests package loader
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-from __future__ import absolute_import
-
 import warnings
 from contextlib import contextmanager
+from urllib.parse import urlsplit
 
-import requests
 import pkg_resources
+import requests
 
-from six import string_types
-from six.moves.urllib.parse import urlsplit
 try:
     from requests.packages.urllib3.exceptions import SSLError
 except ImportError:
@@ -34,7 +30,7 @@ except ImportError:
         from urllib3.exceptions import InsecureRequestWarning  # type: ignore
     except ImportError:
         # for requests < 2.4.0
-        InsecureRequestWarning = None
+        InsecureRequestWarning = None  # type: ignore
 
 try:
     from requests.packages.urllib3.exceptions import InsecurePlatformWarning
@@ -44,11 +40,11 @@ except ImportError:
         from urllib3.exceptions import InsecurePlatformWarning  # type: ignore
     except ImportError:
         # for requests < 2.4.0
-        InsecurePlatformWarning = None
+        InsecurePlatformWarning = None  # type: ignore
 
 # try to load requests[security] (but only if SSL is available)
 try:
-    import ssl
+    import ssl  # NOQA
 except ImportError:
     pass
 else:
@@ -56,24 +52,7 @@ else:
         pkg_resources.require(['requests[security]'])
     except (pkg_resources.DistributionNotFound,
             pkg_resources.VersionConflict):
-        if not getattr(ssl, 'HAS_SNI', False):
-            # don't complain on each url processed about the SSL issue
-            if InsecurePlatformWarning:
-                requests.packages.urllib3.disable_warnings(InsecurePlatformWarning)
-            warnings.warn(
-                'Some links may return broken results due to being unable to '
-                'check the Server Name Indication (SNI) in the returned SSL cert '
-                'against the hostname in the url requested. Recommended to '
-                'install "requests[security]" as a dependency or upgrade to '
-                'a python version with SNI support (Python 3 and Python 2.7.9+).'
-            )
-    except pkg_resources.UnknownExtra:
-        warnings.warn(
-            'Some links may return broken results due to being unable to '
-            'check the Server Name Indication (SNI) in the returned SSL cert '
-            'against the hostname in the url requested. Recommended to '
-            'install requests-2.4.1+.'
-        )
+        pass  # ignored
 
 if False:
     # For type annotation
@@ -108,8 +87,8 @@ def ignore_insecure_warning(**kwargs):
 
 
 def _get_tls_cacert(url, config):
-    # type: (unicode, Config) -> Union[str, bool]
-    """Get addiotinal CA cert for a specific URL.
+    # type: (str, Config) -> Union[str, bool]
+    """Get additional CA cert for a specific URL.
 
     This also returns ``False`` if verification is disabled.
     And returns ``True`` if additional CA cert not found.
@@ -120,7 +99,7 @@ def _get_tls_cacert(url, config):
     certs = getattr(config, 'tls_cacerts', None)
     if not certs:
         return True
-    elif isinstance(certs, (string_types, tuple)):
+    elif isinstance(certs, (str, tuple)):
         return certs  # type: ignore
     else:
         hostname = urlsplit(url)[1]
@@ -131,7 +110,7 @@ def _get_tls_cacert(url, config):
 
 
 def get(url, **kwargs):
-    # type: (unicode, Any) -> requests.Response
+    # type: (str, Any) -> requests.Response
     """Sends a GET request like requests.get().
 
     This sets up User-Agent header and TLS verification automatically."""
@@ -145,7 +124,7 @@ def get(url, **kwargs):
 
 
 def head(url, **kwargs):
-    # type: (unicode, Any) -> requests.Response
+    # type: (str, Any) -> requests.Response
     """Sends a HEAD request like requests.head().
 
     This sets up User-Agent header and TLS verification automatically."""
