@@ -1,11 +1,10 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.util.stemmer
     ~~~~~~~~~~~~~~~~~~~
 
     Word stemming utilities for Sphinx.
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -18,10 +17,10 @@ except ImportError:
     PYSTEMMER = False
 
 
-class BaseStemmer(object):
+class BaseStemmer:
     def stem(self, word):
-        # type: (unicode) -> unicode
-        raise NotImplemented
+        # type: (str) -> str
+        raise NotImplementedError()
 
 
 class PyStemmer(BaseStemmer):
@@ -30,17 +29,17 @@ class PyStemmer(BaseStemmer):
         self.stemmer = _PyStemmer('porter')
 
     def stem(self, word):
-        # type: (unicode) -> unicode
+        # type: (str) -> str
         return self.stemmer.stemWord(word)
 
 
-class StandardStemmer(BaseStemmer, PorterStemmer):  # type: ignore
+class StandardStemmer(PorterStemmer, BaseStemmer):  # type: ignore
     """All those porter stemmer implementations look hideous;
     make at least the stem method nicer.
     """
     def stem(self, word):  # type: ignore
-        # type: (unicode) -> unicode
-        return PorterStemmer.stem(self, word, 0, len(word) - 1)
+        # type: (str) -> str
+        return super().stem(word, 0, len(word) - 1)
 
 
 def get_stemmer():
